%* lsaf_addgroupmember                                                                            *;
%*                                                                                                *;
%* Adds a user or group as a member of a group in the specified organization, project, or         *;
%* analysis.                                                                                      *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_ADDGROUPMEMBER(LSAF_PATH=lsaf-path, LSAF_GROUP=lsaf-group-name,                          *;
%*    LSAF_MEMBER=lsaf-member-identifier <, LSAF_TYPE=lsaf-member-type,                           *;
%*    LSAF_GROUP_CONTEXT=lsaf-group-context>);                                                    *;
%*                                                                                                *;
%* Example Code                                                                                   *;
%* Context paths used in examples:                                                                *;
%* -  Organization: /SAS                                                                          *;
%* -  Project:      /SAS/Project1                                                                 *;
%* -  Analysis:     /SAS/Project1/Analysis1                                                       *;
%*                                                                                                *;
%* 1. Add User as a Group member at the Project level:                                            *;
%*    %lsaf_addgroupmember(lsaf_path=%str(/SAS/Project1), lsaf_group=%str(PrjGrp1),               *;
%*    lsaf_member=%str(User1), lsaf_type=%str(USER));                                             *;
%*                                                                                                *;
%* 2. Add Group as a Group member at the Project level:                                           *;
%*    %lsaf_addgroupmember(lsaf_path=%str(/SAS/Project1), lsaf_group=%str(PrjGrp1),               *;
%*    lsaf_member=%str(OrgGrp1), lsaf_type=%str(GROUP), lsaf_group_context=%str(/SAS));           *;
%*                                                                                                *;
%* 3. Add User as a Group member at the Analysis level:                                           *;
%*    %lsaf_addgroupmember(lsaf_path=%str(/SAS/Project1/Analysis1),                               *;
%*    lsaf_group=%str(AnaGrp1), lsaf_member=%str(User1), lsaf_type=%str(USER));                   *;
%*                                                                                                *;
%* 4. Add Project Group as a Group member at the Analysis level:                                  *;
%*    %lsaf_addgroupmember(lsaf_path=%str(/SAS/Project1/Analysis1),                               *;
%*    lsaf_group=%str(AnaGrp1), lsaf_member=%str(PrjGrp1), lsaf_type=%str(GROUP),                 *;
%*    lsaf_group_context=%str(/SAS/Project1));                                                    *;
%*                                                                                                *;
%* 5. Add Organization Group as a Group member at the Analysis level:                             *;
%*    %lsaf_addgroupmember(lsaf_path=%str(/SAS/Project1/Analysis1),                               *;
%*    lsaf_group=%str(AnaGrp1), lsaf_member=%str(OrgGrp1), lsaf_type=%str(GROUP),                 *;
%*    lsaf_group_context=%str(/SAS));                                                             *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the context object (organization, project, or        *;
%*            analysis) where the group is defined.                                               *;
%* @param lsaf_group - required - the name of the group to add the member.                        *;
%* @param lsaf_member - required - the user ID of the user or the name of the group being added.  *;
%* @param lsaf_type - conditional - the type of member being added.                               *;
%*            Values:  USER | GROUP (case-insensitive)                                            *;
%*            Default: USER                                                                       *;
%* @param lsaf_group_context - conditional - specifies the context in which the group is defined  *;
%*            as a path (e.g., /SAS). This parameter is required if LSAF_TYPE=GROUP. Otherwise,   *;
%*            it will be ignored.                                                                 *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-29  rebrand as LSAF                                                           *;
%*                                                                                                *;

%macro lsaf_addgroupmember(
  lsaf_path=,
  lsaf_group=,
  lsaf_type=USER,
  lsaf_member=,
  lsaf_group_context=
  ) /des='Add a member to the group in the specified context.';


  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix     length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/GroupService") ;
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("addMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_group)", "%bquote(&lsaf_type)", "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", results );

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The &lsaf_type &lsaf_member has been added to &lsaf_group in &lsaf_path.." ;
    end;
      else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot add &lsaf_type &lsaf_member to &lsaf_group in &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_addgroupmember;
