%* lsaf_adduserprivilege                                                                         *;
%*                                                                                               *;
%* Assign a global privilege to a user in the system.                                            *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_ADDUSERPRIVILEGE(LSAF_USERID=lsaf-user-id, LSAF_PRIVILEGE=lsaf-privilege-id);           *;
%*                                                                                               *;
%* Example Code:                                                                                 *;
%* Use %lsaf_getglobalprivileges to fetch the list of available global privileges. The           *;
%* macro returns a SAS data set containing the list of privileges (the default name of the       *;
%* data set is work.lsafgetglobalprivileges).  Use one of the privilegeId values to assign that  *;
%* global privilege to a User.                                                                   *;
%*                                                                                               *;
%* 1. Retrieve the available global privileges:                                                  *;
%*    %lsaf_getglobalprivileges();                                                               *;
%*                                                                                               *;
%* 2. Assign a privilege to a User:                                                              *;
%*    %lsaf_adduserprivilege(lsaf_userid=%str(User1),                                            *;
%*    lsaf_privilege=%str(PRIVILEGE_AUDIT_REPORTING));                                           *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_userid - required - the user to assign the global privilege.                      *;
%* @param lsaf_privilege - required - the identifier of the global privilege to assign.          *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.3                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-11-01 initial coding                                                            *;
%*                                                                                               *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;

%macro lsaf_adduserprivilege(
  lsaf_userid=,
  lsaf_privilege=
  ) /des='Assign a global privilege to a user.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/PrivilegeService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("addPrivilege", "%bquote(&lsaf_userid)", "%bquote(&lsaf_privilege)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The global privilege &lsaf_privilege has been assigned to the user &lsaf_userid.." ;
    end;
    else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot assign the global privilege &lsaf_privilege to the user &lsaf_userid.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_adduserprivilege;
