%* lsaf_assignedroleexists                                                                        *;
%*                                                                                                *;
%* Determine if a role exists (as defined or inherited) in an organization, project, or analysis. *;
%*                                                                                                *;
%* The SAS macro variable _lsafAssignedRoleExists_ is set to a value of 1 if the role exists, 0   *;
%* if it does not. In the event of an incorrect call to the macro which results in an error, the  *;
%* return value of _lsafAssignedRoleExists_ will be set to -1.                                    *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_ASSIGNEDROLEEXISTS (LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                       *;
%* LSAF_ROLE_CONTEXT=lsaf-role-context);                                                          *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%* @macvar _lsafAssignedRoleExists_  indicates the existence of the assigned role in the context. *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the context object (organization, project, or        *;
%*            analysis).                                                                          *;
%* @param lsaf_role - required - the name of the role to check for existence.                     *;
%* @param lsaf_role_context - required - specifies the context in which the role is defined as a  *;
%*            path (e.g., /SAS).                                                                  *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%*                                                                                                *;

%macro lsaf_assignedroleexists(
  lsaf_path=,
  lsaf_role=,
  lsaf_role_context=
  ) /des='Is the role assigned to a context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafAssignedRoleExists_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RoleService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("assignedRoleExists", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_role_context)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the assignment of the role &lsaf_role from context &lsaf_role_context in &lsaf_path..";
      exists=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (exists) then
      do;
        logMsg=" The role &lsaf_role from context &lsaf_role_context is assigned in &lsaf_path.." ;
      end;
      else
      do;
        logMsg=" The role &lsaf_role from context &lsaf_role_context is not assigned in &lsaf_path.." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafAssignedRoleExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafAssignedRoleExists_", exists) ;

  run;

%mend lsaf_assignedroleexists ;
