%* lsaf_checkin                                                                                  *;
%*                                                                                               *;
%* Checks in a SAS Life Science Analytics Framework repository file.                             *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_CHECKIN(LSAF_PATH=lsaf-path                                                             *;
%*            <,LSAF_COMMENT=lsaf-comment                                                        *;
%*            ,LSAF_VERSIONTYPE=lsaf-versiontype                                                 *;
%*            ,LSAF_CUSTOMVERSION=lsaf-customversion>));                                         *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the file within the SAS Life Science Analytics      *;
%*            Framework repository to check in.                                                  *;
%* @param lsaf_comment - optional - a note associated with the action of checking in the file.   *;
%* @param lsaf_versiontype - optional - the version type (MINOR, MAJOR, or CUSTOM) to be created *;
%*            by the checkin action. The default value is MINOR.                                 *;
%* @param lsaf_customversion - conditional - a specific version to be created by the checkin     *;
%*            action. The parameter is required when lsaf_versiontype is set to CUSTOM.          *;
%*            Otherwise, it will be ignored.                                                     *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-10-05 initial coding                                                            *;
%*                                                                                               *;

%macro lsaf_checkin(
  lsaf_path=,
  lsaf_comment=,
  lsaf_versiontype=MINOR,
  lsaf_customversion=
  ) /des='Allows check in of a SAS Life Science Analytics Framework repository file.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("checkIn","%bquote(&lsaf_path)","%bquote(&lsaf_comment)",
                             "%bquote(&lsaf_versiontype)","%bquote(&lsaf_customversion)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Unable to check in the file: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Check in successful for &lsaf_path..";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_checkin;
