%* lsaf_createfile                                                                                 *;
%*                                                                                                 *;
%* Uploads a file from the local machine to the SAS Life Science Analytics Framework repository.   *;
%*                                                                                                 *;
%* All parent folders that do not already exist will be created as well.                           *;
%*                                                                                                 *;
%* NOTE: This macro is only supported from a PC SAS session. Using this macro within an active     *;
%* SAS Life Science Analytics Framework application session will result in a failure.              *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_CREATEFILE(LOCAL_PATH=local-path, LSAF_PATH=lsaf-path                                     *;
%* <, LSAF_VERSIONING=lsaf-versioning, LSAF_VERSION=lsaf-version, LSAF_COMMENT=lsaf_comment>);     *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param local_path - required - the absolute path and name of the file on the local computer.    *;
%* @param lsaf_path - required - the path of the file to be created. Files can only be created in  *;
%*            SAS Life Science Analytics Framework areas that support folder and file creation.    *;
%*            They cannot be created at the organization, project, or analysis levels.             *;
%* @param lsaf_versioning - optional - indicates whether the file being created should be          *;
%*            versioned. The default value is 0, which will create a non-versioned file.  A        *;
%*            value of 1 will create a versioned file.                                             *;
%*            Values:  0 | 1                                                                       *;
%*            Default: 0                                                                           *;
%* @param lsaf_version - conditional - indicates the version number to assign to the file being    *;
%*            created. The value must be in the valid version label format including a decimal     *;
%*            point.  This option is ignored when creating non-versioned file.                     *;
%* @param lsaf_comment - conditional - the comment for the versioned file being created. This      *;
%*            option is ignored when creating a non-versioned file.                                *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.1                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-29  rebrand as LSAF                                                            *;
%*                                                                                                 *;

%macro lsaf_createfile(
  local_path=,
  lsaf_path=,
  lsaf_versioning=,
  lsaf_version=,
  lsaf_comment=
  ) /des='Upload a file from the local computer as a new file into SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("createFile", "%bquote(&local_path)", "%bquote(&lsaf_path)", "%bquote(&lsaf_versioning)", "%bquote(&lsaf_version)", "%bquote(&lsaf_comment)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if ("&lsaf_versioning" eq "1") then
      do;
        logMsg=" Versioned file created as version &lsaf_version at &lsaf_path";
      end;
      else
      do;
        logMsg=" Unversioned file was created at &lsaf_path";
      end;
    end;
    else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" The file was not created: &lsaf_path" ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " local_path= &local_path" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_createfile ;
