%* lsaf_createprocessflow                                                                        *;
%*                                                                                               *;
%* Creates a process flow in the SAS Life Science Analytics Framework repository. A process flow *;
%* can only be created at the repository, project, or analysis level and not at a folder level.  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_CREATEPROCESSFLOW(LSAF_PATH=lsaf-path,                                                  *;
%*                         LSAF_processflow=process-flow-name,                                   *;
%*                         LSAF_processflowdef=process-flow-definition-name                      *;
%*                         );                                                                    *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the process flow being created.                     *;
%* @param lsaf_processflow - required - the name of the process flow being created.              *;
%* @param lsaf_processflowdef - required - the name of the process flow definition               *;
%*                          used to create the process flow.                                     *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-08-30  initial programming                                                      *;
%*                                                                                               *;

%macro lsaf_createprocessflow(
  lsaf_path=,
  lsaf_processflow=,
  lsaf_processflowdef=
  ) /des='Create a process flow in the specified SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("create", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", "%bquote(&lsaf_processflowdef)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot create process flow &lsaf_processflow in &lsaf_path.." ;
    end;
	else
	do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The process flow &lsaf_processflow was created in &lsaf_path.." ;
	end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_createprocessflow;
