%* lsaf_createprocessflowmanifest                                                                 *;
%*                                                                                                *;
%* Creates a process flow manifest file in the SAS Life Science Analytics Framework repository.   *;
%* The SAS macro variable _lsafCreatedProcessFlowManifest_ is set to the full path of the         *;
%* process flow manifest including the file extension. If the process flow manifest cannot be     *;
%* created, _lsafCreatedProcessFlowManifest_ will be set to blank.                                *;
%*                                                                                                *;
%* If the manifest file already exists, a new version of the file will be created. If the         *;
%* existing file is not versioned, it will be overwritten by the new file.                        *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_CREATEPROCESSFLOWMANIFEST(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name,       *;
%* LSAF_MANIFESTPATH=manifest-path <, LSAF_VERSIONING=lsaf-versioning,                            *;
%* LSAF_VERSIONTYPE=lsaf-version-type, LSAF_CUSTOMVERSION=lsaf-custom-version,                    *;
%* LSAF_COMMENT=lsaf-comment>);                                                                   *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%* @macvar _lsafCreatedProcessFlowManifest_ indicates the full path of the process flow manifest. *;
%*                                                                                                *;
%* @param lsaf_path - required - the path where the process flow exists.                          *;
%* @param lsaf_processflow - required - the name of the process flow.                             *;
%* @param lsaf_manifest_path - required - the repository location for the manifest. The location  *;
%*            must be a folder and must already exist in the repository.                          *;
%* @param lsaf_versioning - optional - indicates whether the file being created should be         *;
%*            versioned. The default value is 1, which will create a versioned file.  A value     *;
%*            of 0 will create an unversioned file.                                               *;
%* @param lsaf_versiontype - optional - the version type (MINOR, MAJOR, or CUSTOM) to be          *;
%*            created if the file is to be versioned. The default value is MAJOR. The value is    *;
%*            ignored for unversioned files.                                                      *;
%* @param lsaf_customversion - conditional - indicates the version number to assign to the file   *;
%*            being created. The value must be in the valid version label format including a      *;
%*            decimal point. This option is required when versiontype is CUSTOM and is ignored    *;
%*            when creating a non-versioned file.                                                 *;
%* @param lsaf_comment - optional - The comment will display with the audit trail record for the  *;
%*            manifest creation.                                                                  *;
%*                                                                                                *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-11-15  initial programming                                                       *;
%*                                                                                                *;

%macro lsaf_createprocessflowmanifest(
  lsaf_path=,
  lsaf_processflow=,
  lsaf_manifest_path=,
  lsaf_versioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Create a manifest file for the specifed process flow in the indicated SAS Life Science Analytics Framework location.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafCreatedProcessFlowManifest_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode     length=8
           message        length=$200
           logMsgPrefix   length=$70
           logMsg         length=$300
           manifestPath   length=$1024
    ;
    manifestPath="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("createManifest", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", "%bquote(&lsaf_manifest_path)",
                             "%bquote(&lsaf_versioning)", "%bquote(&lsaf_versiontype)", "%bquote(&lsaf_customversion)", "%bquote(&lsaf_comment)", results, manifestPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot create a manifest for process flow &lsaf_processflow in &lsaf_manifest_path.." ;
    end;
   else
   do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The manifest for process flow &lsaf_processflow was created in &lsaf_manifest_path.." ;
   end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafCreatedProcessFlowManifest_= " manifestPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafCreatedProcessFlowManifest_", manifestPath);

  run;

%mend lsaf_createprocessflowmanifest;
