%* lsaf_deleteprocessflow                                                                         *;
%*                                                                                                *;
%* Deletes a process flow and, optionally, creates manifest file in the SAS Life Science          *;
%* Analytics Framework repository. The SAS macro variable _lsafDeletedProcessFlowManifest_ is set *;
%* to the full path of the process flow manifest including the file extension. If no process      *;
%* flow manifest is requested, _lsafDeletedProcessFlowManifest_ will be set to blank. If a        *;
%* process flow manifest is requested but cannot be created, _lsafDeletedProcessFlowManifest_     *;
%* will be set to blank and the process flow will not be deleted.                                 *;
%*                                                                                                *;
%* If the manifest file already exists, a new version of the file will be created. If the         *;
%* existing file is not versioned, it will be overwritten by the new file.                        *;
%*                                                                                                *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_DELETEPROCESSFLOW(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name,               *;
%*                <, LSAF_CREATEMANIFEST=lsaf-createmanifest, LSAF_MANIFESTPATH=manifest-path,    *;
%*                   LSAF_VERSIONING=lsaf-versioning, LSAF_VERSIONTYPE=lsaf-version-type,         *;
%*                   LSAF_CUSTOMVERSION=lsaf-custom-version, LSAF_COMMENT=lsaf-comment>);         *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%* @macvar _lsafDeletedProcessFlowManifest_ indicates the full path of the process flow manifest. *;
%*                                                                                                *;
%* @param lsaf_path - required - the path where the process flow exists.                          *;
%* @param lsaf_processflow - required - the name of the process flow.                             *;
%* @param lsaf_createmanifest - optional - indicates whether to create a process flow manifest    *;
%*            file prior to deleting the process flow. The default value is 1 indicating to       *;
%*            create a manifest.                                                                  *;
%* @param lsaf_manifest_path - optional - the repository location for the manifest. The location  *;
%*             must be a folder and must already exist in the repository. This parameter is       *;
%*             required when lsaf_createmanifest is set to 1. Otherwise, it will be ignored.      *;
%* @param lsaf_versioning - optional - indicates whether the manifest should be versioned. The    *;
%*            default value is 1, which will create a versioned file.  A value of 0 will create   *;
%*            an unversioned file.                                                                *;
%* @param lsaf_versiontype - optional - the version type (MINOR, MAJOR, or CUSTOM) to be          *;
%*            created if the file is to be versioned. The default value is MAJOR. The value is    *;
%*            ignored for unversioned files or if lsaf_createmanifest is set to 0.                *;
%* @param lsaf_customversion - conditional - indicates the version number to assign to the file   *;
%*            being created. The value must be in the valid version label format including a      *;
%*            decimal point. This parameter is required when versiontype is CUSTOM and is ignored *;
%*            when creating a non-versioned file or if lsaf_createmanifest is set to 0.           *;
%* @param lsaf_comment - optional - The comment will display with the audit trail record for      *;
%*            the manifest creation.                                                              *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-11-16  initial programming                                                       *;
%*                                                                                                *;

%macro lsaf_deleteprocessflow(
  lsaf_path=,
  lsaf_processflow=,
  lsaf_createmanifest=1,
  lsaf_manifest_path=,
  lsaf_versioning=,
  lsaf_versiontype=,
  lsaf_customversion=,
  lsaf_comment=
  ) /des='Delete a process flow and optionally create a manifest file';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafDeletedProcessFlowManifest_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode     length=8
           message        length=$200
           logMsgPrefix   length=$70
           logMsg         length=$300
           manifestPath   length=$1024
    ;
    manifestPath="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("delete", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", "%bquote(&lsaf_createmanifest)",
                                         "%bquote(&lsaf_manifest_path)", "%bquote(&lsaf_versioning)", "%bquote(&lsaf_versiontype)",
                                         "%bquote(&lsaf_customversion)", "%bquote(&lsaf_comment)", results, manifestPath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot delete the process flow &lsaf_processflow.." ;
    end;
   else
   do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The process flow &lsaf_processflow was deleted." ;
   end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    if (kstrip(manifestPath) ne "") then
    do;
      put logMsgPrefix " Created manifest file: " manifestPath;
    end;
    else
    do;
      put logMsgPrefix " No manifest file was created.";
    end;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafDeletedProcessFlowManifest_= " manifestPath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafDeletedProcessFlowManifest_", manifestPath);

  run;

%mend lsaf_deleteprocessflow;
