%* lsaf_getobjecttype                                                                            *;
%*                                                                                               *;
%* Reports the SAS Life Science Analytics Framework object type for the specified                *;
%* repository object path.                                                                       *;
%*                                                                                               *;
%* The SAS macro variable _lsafObjectType_ is set to the object type value.                      *;
%* The value is set to blank if the object could not be found or an error occurred.              *;
%* Possible object type values are:                                                              *;
%*    sas:analysis                                                                               *;
%*    sas:container                                                                              *;
%*    sas:file                                                                                   *;
%*    sas:folder                                                                                 *;
%*    sas:job                                                                                    *;
%*    sas:organization                                                                           *;
%*    sas:pdf                                                                                    *;
%*    sas:project                                                                                *;
%*    sas:sasdataset                                                                             *;
%*    sas:sasprogram                                                                             *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETOBJECTTYPE(LSAF_PATH=lsaf-path);                                                     *;
%*                                                                                               *;
%* @macvar _lsafRC_         the return code.                                                     *;
%* @macvar _lsafMsg_        the return message.                                                  *;
%* @macvar _lsafObjectType_ the object type.                                                     *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the object for which the object type                *;
%*                   will be retrieved.                                                          *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-03-07  initial coding                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_getobjecttype(
  lsaf_path=
  ) /des='Get the SAS Life Science Analytics Framework object type for the specified path.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafObjectType_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib objectType      length=$30;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getObjectType", "%bquote(&lsaf_path)", results, objectType);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not determine the object type: &lsaf_path." ;
      objectType="";
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The object type was returned for path: &lsaf_path";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix logMsg;
    put logMsgPrefix "_lsafMsg_=" message ;
    put logMsgPrefix "_lsafRC_=" returnCode ;
   put logMsgPrefix "_objectType_=" objectType ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafObjectType_", objectType) ;
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_getobjecttype ;
