%* lsaf_getowner                                                                                 *;
%*                                                                                               *;
%* Returns the current owner of an object within the organization in the SAS Life Science        *;
%* Analytics Framework repository.                                                               *;
%*                                                                                               *;
%* The macro variable _lsafOwner_ will be set with the owner user ID. The value of _lsafOwner_   *;
%* will be <creator> for the DEFAULT permissions on a container with no specific user designated *;
%* as the owner.                                                                                 *;
%*                                                                                               *;
%* Syntax for files:                                                                             *;
%* %LSAF_GETOWNER(LSAF_PATH =lsaf-path);                                                         *;
%*                                                                                               *;
%* Syntax For containers:                                                                        *;
%* %LSAF_GETOWNER(LSAF_PATH =lsaf-path, LSAF_ACLTYPE =lsaf-acl-type);                            *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafOwner_ the current owner of the object.                                          *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the content object in the SAS Life Science          *;
%*            Analytics Framework repository.                                                    *;
%* @param lsaf_aclType - conditional - the type of permissions being returned.  The valid values *;
%*            are DEFAULT and CURRENT (case-insensitive). Files only have CURRENT permissions,   *;
%*            therefore, the value is not required when lsaf_path is a file. For container       *;
%*            objects the value must be specified. The operation will fail if DEFAULT is         *;
%*            requested for a file path.                                                         *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_getowner(
  lsaf_path=,
  lsaf_aclType=
  ) /des='Get the owner of an object.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafOwner_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    attrib owner           length=$64;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/AclService") ;
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getOwner", "%bquote(&lsaf_path)", "%bquote(&lsaf_aclType)", results, owner);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve owner of the &lsaf_aclType ACL of &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The owner of the &lsaf_aclType ACL of &lsaf_path is " || owner ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafOwner_= " owner;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafOwner_", owner) ;

  run;

%mend lsaf_getowner;
