%* lsaf_getpfsignallocations                                                                       *;
%*                                                                                                 *;
%* Returns a SAS data set that contains the locations for the signal setup elements for the        *;
%* process flow within the specified organization, project, or analysis.                           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPFSIGNALLOCATIONS(LSAF_PATH=lsaf-path,                                                 *;
%*     LSAF_PROCESSFLOW=process-flow-name <,SAS_DSNAME=SAS-data-set>);                             *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) where the process flow exists.                                             *;
%* @param lsaf_processflow - required - the name of the process flow.                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            locations for the process flow signal element specified as libref.dataset.           *;
%*            The default value is WORK.LSAFGETPFSIGNALLOCATIONS. The data set contains a          *;
%*            row for each signal location as well as columns with the following names. The        *;
%*            data set will be sorted by element id and location.                                  *;
%*            The following metadata will be represented in the data set.                          *;
%*            - processFlowPath: The path of the context object.                                   *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - elementID: The id for the element applicable for the parameter.                    *;
%*            - location: The path for the signal location.                                        *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-14  original creation.                                                         *;
%*                                                                                                 *;

%macro lsaf_getpfsignallocations(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafGetPfSignalLocations
  ) /des='Get a list of process flow signal setup element locations for the specific process flow.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %let notesOption=%sysfunc(getoption(notes));
  options notes;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= processFlowPath processFlowName elementID location);

    attrib returnCode          length=8
           message             length=$200
           logMsgPrefix        length=$70
           logMsg              length=$300
           rowCount            length=8

           processFlowPath     length=$2048  label="Context Path"
           processFlowName     length=$256   label="Process Flow Name"
           elementID           length=$256   label="Element ID"

           location            length=$2048  label="Signal Location Path"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService") ;
    declare javaobj results("com/sas/hls/macro/results/SasProcessFlowSignalLocationsResult");

    %***********************************;
    %* Attempt the operation           *:
    %***********************************;
    srv.callStaticVoidMethod("getSignalLocations", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)",
         results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow signal element locations for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Process flow signal element locations were returned for &lsaf_processflow in &lsaf_path" ;

      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getElementID", row, elementID);

        results.callStringMethod("getPath", row, location);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No process flow signal elements locations were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";

    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    options nonotes;
    proc sort data=&sas_dsname;
      by elementID location;
    run;
    options &notesOption;
  %end;


%mend lsaf_getpfsignallocations;
