%* lsaf_getpfuserelementinfo                                                                       *;
%*                                                                                                 *;
%* Returns a SAS data set that contains the properties for the user setup elements for the         *;
%* process flow within the specified organization, project, or analysis.                           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPFUSERELEMENTINFO(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name              *;
%* <, SAS_DSNAME=SAS-data-set>);                                                                   *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) where the process flow exists.                                             *;
%* @param lsaf_processflow - required - the name of the process flow.                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            properties and attributes for the process flow data specified as libref.dataset.     *;
%*            The default value is WORK.LSAFGETPFUSERELEMENTINFO. The data set contains a          *;
%*            row for each property or attribute as well as columns with the following names. The  *;
%*            data set is not sorted.                                                              *;
%*            The following metadata will be represented in the data set.                          *;
%*            - processFlowPath: The path of the context object.                                   *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - elementID: The id for the element applicable for the property.                     *;
%*            - name: The name of the property.                                                    *;
%*            - value: The value of the property.                                                  *;
%*            - displayName: The name of the property as displayed as a label within SAS Life      *;
%*              Science Analytics Framework. Derived properties (e.g. name) will not have a value  *;
%*              value for displayName.                                                             *;
%*            - type: The type of the property. Values include STRING, LONG, DATE, and BOOLEAN.    *;
%*            - isReadonly: Flag indicating that the property cannot be modified.                  *;
%*            - isExtended: Flag indicating whether the property is an extended attribute.         *;
%*                                                                                                 *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-08  original creation.                                                         *;
%*                                                                                                 *;

%macro lsaf_getpfuserelementinfo(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafGetPfUserElementInfo
  ) /des='Get a list of process flow user setup element properties for the specific process flow.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;
  %let notesOption=%sysfunc(getoption(notes));
  options notes;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= processFlowPath processFlowName elementID
                          name displayName value type isReadonly isExtended);

    attrib returnCode          length=8
           message             length=$200
           logMsgPrefix        length=$70
           logMsg              length=$300
           rowCount            length=8

           processFlowPath     length=$2048  label="Context Path"
           processFlowName     length=$256   label="Process Flow Name"
           elementID           length=$256   label="Element ID"

           name            length=$256     label="Property Name"
           displayName     length=$256     label="Property Display Name"
           value           length=$2048    label="Property Value"
           type            length=$32      label="Property Type"
           isReadonly      length=8        label="Property value is not modifiable."
           isExtended      length=8        label="Property is an extended attribute"
    ;


    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService") ;
    declare javaobj results("com/sas/hls/macro/results/SasProcessFlowPropertiesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getElementPropertiesByType", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)",
        "%bquote(USER_TASK)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow user element properties for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Process flow user element properties were returned for &lsaf_processflow in &lsaf_path" ;

      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getElementID", row, elementID);
        results.callStringMethod("getPropertyName", row, name);
        results.callStringMethod("getPropertyValue", row, value);
        results.callStringMethod("getPropertyDisplayName", row, displayName);
        results.callStringMethod("getPropertyType", row, type);
        results.callBooleanMethod("getPropertyIsReadonly", row, isReadonly);
        results.callBooleanMethod("getPropertyIsExtended", row, isExtended);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No process flow user elements properties were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";

    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;
  options &notesOption;

%mend lsaf_getpfuserelementinfo;
