%* lsaf_getprocessdefsbytype                                                                       *;
%*                                                                                                 *;
%* Returns a SAS data set that contains the metadata for all of the process flow definitions       *;
%* available at the specified context type.                                                        *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPROCESSDEFSBYTYPE(LSAF_TYPE=lsaf-type <,SAS_DSNAME=SAS-data-set>);                     *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_type - optional - the context type. Valid values are ORGANIZATION, PROJECTS, or     *;
%*            ANALYSES. The default value is ORGANIZATION.                                         *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            metadata for the process flow definitions specified as libref.dataset. The default   *;
%*            value is WORK.LSAFGETPROCESSDEFSBYTYPE. The data set contains a row for each         *;
%*            process flow definition deployed at context level as well as columns with the        *;
%*            following names.                                                                     *;
%*            The data set will be sorted by process flow definition ID.                            *;
%*            - contexts: A comma delimited string listing the context types the process flow      *;
%*              definition is deployed.                                                            *;
%*            - defID: The ID of the process flow definition.                                      *;
%*            - name: The name of the process flow definition.                                     *;
%*            - description: The process flow description.                                         *;
%*            - deployedVersion: The version of the process flow definition.                       *;
%*            - filePath: The path of the source file from which the process flow definition was   *;
%*              deployed.                                                                          *;
%*            - fileVersion: The version of the source file from which process flow definition     *;
%*              deployed.                                                                          *;
%*            - isAtOrganization: A numeric flag that indicates whether the process flow           *;
%*              definition is deployed at the organization level. (0=no, 1=yes)                    *;
%*            - isAtProjects: A numeric flag that indicates whether the process flow               *;
%*              definition is deployed at the projects level. (0=no, 1=yes)                        *;
%*            - isAtAnalyses: A numeric flag that indicates whether the process flow               *;
%*              definition is deployed at the analyses level. (0=no, 1=yes)                        *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-08-23  original creation.                                                         *;

%macro lsaf_getprocessdefsbytype(
  lsaf_type=ORGANIZATION,
  sas_dsname=work.lsafGetProcessDefsByType
  ) /des='Get a list of process flow definitions deployed at the specified context type.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contexts defId name description deployedVersion sourceFilePath sourceFileVersion
                         isAtOrganization isAtProjects isAtAnalyses)
  ;

    attrib returnCode          length=8
           message             length=$200
           logMsgPrefix        length=$70
           logMsg              length=$300
           rowCount            length=8

           defID               length=$256   label="Process Flow Definition ID"
           name                length=$256   label="Process Flow Definition Name"
           description         length=$1024  label="Process Flow Definition Description"

           deployedVersion     length=$32    label="Process Flow Definition Version"

           sourceFilePath      length=$2048  label="Source file path"
           sourceFileVersion   length=$32    label="Source file version number"

           contexts            length=$64    label="Available contexts"

           isAtOrganization    length=8      label="Is Process Flow Definition deployed at the Organization level?"
           isAtProjects        length=8      label="Is Process Flow Definition deployed at the Projects level?"
           isAtAnalyses        length=8      label="Is Process Flow Definition deployed at the Analyses level?"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService") ;
    declare javaobj results("com/sas/hls/macro/results/SasProcessDefinitionInfoResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getProcessDefinitionsByType", "%bquote(&lsaf_type)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow definitions for type &lsaf_type" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Process flow definitions were returned for &lsaf_type" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getId", row, defID);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getDeployedVersion", row, deployedVersion);

        results.callStringMethod("getDefinitionFilePath", row, sourceFilePath);
        results.callStringMethod("getDefinitionFileVersion", row, sourceFileVersion);

        results.callStringMethod("getContexts", row, contexts);

        results.callIntMethod("isAvailableToOrg", row, isAtOrganization);
        results.callIntMethod("isAvailableToProjects", row, isAtProjects);
        results.callIntMethod("isAvailableToAnalyses", row, isAtAnalyses);

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No process flow definitions were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by defID;
    run;
    options &notesOption;
  %end;


%mend lsaf_getprocessdefsbytype;
