%* lsaf_getprocessflowdata                                                                         *;
%*                                                                                                 *;
%* Returns a SAS data set that contains the metadata for the process data for the specified        *;
%* process flow within the specified organization, project, or analysis.                           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPROCESSFLOWDATA(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name                *;
%*              <,SAS_DSNAME=SAS-data-set>);                                                       *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) where the process flow exists.                                             *;
%* @param lsaf_processflow - required - the name of the process flow.                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            metadata for the process flow data specified as libref.dataset. The default value is *;
%*            WORK.LSAFGETPROCESSFLOWDATA. The data set contains a row for each process flow data  *;
%*            element as well as columns with the following names. The data set is not sorted.     *;
%*            - processFlowPath: The path of the context object.                                   *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - dataID: The ID of the process flow data element.                                   *;
%*            - dataName: The name of the process flow data element.                               *;
%*            - dataType: The type of the process flow data element.                               *;
%*            - dataValue: The value of the process flow data element represented as a string.     *;
%*            - dataEnumValues: A comma-delimited list of the defined values for the elements of   *;
%*              the type ENUM displayed in the format key(value).                                  *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-11-22  original creation.                                                         *;
%*                                                                                                 *;

%macro lsaf_getprocessflowdata(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafGetProcessFlowData
  ) /des='Get a list of process flow data of a specific process flow in a SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=processFlowPath processFlowName
                         dataID dataName dataType dataValue dataEnumValues)
  ;


    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsg                    length=$300
           rowCount                  length=8

           processFlowPath           length=$2048  label="Context Path"
           processFlowName           length=$256   label="Process Flow Name"

           dataID                    length=$256   label="Process Data Element Identifier"
           dataName                  length=$256   label="Process Data Element Name"
           dataType                  length=$32    label="Process Data Element Type"
           dataValue                 length=$1024  label="Process Data Element Value"
           dataEnumValues            length=$5000  label="Process Data Element Enum Valid Values"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService") ;
    declare javaobj results("com/sas/hls/macro/results/SasProcessFlowDataResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getProcessData", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process data for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Process data were returned for &lsaf_processflow in &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);

        results.callStringMethod("getId", row, dataID);
        results.callStringMethod("getName", row, dataName);
        results.callStringMethod("getType", row, dataType);
        results.callStringMethod("getValue", row, dataValue);
        results.callStringMethod("getEnumValues", row, dataEnumValues);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No process flow data were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

%mend lsaf_getprocessflowdata;
