%* lsaf_getprocessflowelements                                                                     *;
%*                                                                                                 *;
%* Returns a SAS data set that contains the metadata for the flow setup elements for the           *;
%* specified process flow within the specified organization, project, or analysis.                 *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETPROCESSFLOWELEMENTS(LSAF_PATH=lsaf-path, LSAF_processflow=process-flow-name            *;
%*              <,SAS_DSNAME=SAS-data-set>);                                                       *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) where the process flow exists.                                             *;
%* @param lsaf_processflow - required - the name of the process flow.                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            elements for the process flow data specified as libref.dataset.                      *;
%*            The default value is WORK.LSAFGETPROCESSFLOWELEMENTS.  The data set contains a       *;
%*            row for each element as well as columns with the following names. The data set is    *;
%*            not sorted.                                                                          *;
%*            The following metadata will be represented in the data set.                          *;
%*            - processFlowPath: The path of the context object.                                   *;
%*            - processFlowName: The name of the process flow.                                     *;
%*            - elementType: The type of the element.                                              *;
%*            - elementID: The unique ID  of the element.                                          *;
%*            - elementName: The name of the element.                                              *;
%*            - elementDescription: The description of the element.                                *;
%*                                                                                                 *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-12-07  original creation.                                                         *;
%*                                                                                                 *;

%macro lsaf_getprocessflowelements(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=work.lsafGetProcessFlowElements
  ) /des='Get a list of process flow setup elements of a specific process flow in a SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep= processFlowPath processFlowName
                          elementID elementName elementDescription elementType);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsg                    length=$300
           rowCount                  length=8

           processFlowPath           length=$2048   label="Context Path"
           processFlowName           length=$256    label="Process Flow Name"

           elementType               length=$64     label="Element Type"
           elementID                 length=$256    label="Element Name"
           elementName               length=$256    label="Element Display Name"
           elementDescription        length=$2048   label="Element Value"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService") ;
    declare javaobj results("com/sas/hls/macro/results/SasProcessFlowElementsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getAllSetupElements", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve process flow elements for &lsaf_processflow in &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Process flow elements were returned for &lsaf_processflow in &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getProcessFlowContext", processFlowPath);
        results.callStringMethod("getProcessFlowName", processFlowName);
        results.callStringMethod("getId", row, elementID);
        results.callStringMethod("getName", row, elementName);
        results.callStringMethod("getDescription", row, elementDescription);
        results.callStringMethod("getType", row, elementType);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No process flow elements were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

%mend lsaf_getprocessflowelements;
