%* lsaf_getrbitemsbyquery                                                                        *;
%*                                                                                               *;
%* Returns a SAS data set that contains the metadata for the items in the SAS Life Science       *;
%* Analytics Framework recycle bin that match the given search criteria.                         *;
%*                                                                                               *;
%* Users must have the Permanent Delete Items or Restore Items Deleted by Others role privilege  *;
%* at the organization context.                                                                  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETRBITEMSBYQUERY(LSAF_PATH=lsaf-path <, LSAF_NAME=lsaf-name, LSAF_USERID-lsaf-userid,  *;
%*         LSAF_FROM=lsaf-from-date, LSAF_TO=lsaf-to-date, SAS_DSNAME=SAS-data-set>);            *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the container object in the SAS Life Science        *;
%*            Analytics Framework repository that is the parent of the deleted items. Items      *;
%*            from child paths will also be returned. For example, providing a project path will *;
%*            return items that were deleted from the project and any child analyses.            *;
%* @param lsaf_name - optional - The name of the item to be returned by the search. If set to    *;
%*            * (ie lsaf_name=*) or omitted, items of any name will be returned.                 *;
%*            Additionally, wildcard notation can be used, for example lsaf_name=*.sas will      *;
%*            return all items with a .sas extension.                                            *;
%* @param lsaf_userid - optional - The id of the user who deleted the items to be retrieved.     *;
%*            If omitted or set to blank, the items deleted by the person running the program    *;
%*            will be returned. If set to * (ie lsaf_userid=*), the items deleted from any user  *;
%*            will be returned.                                                                  *;
%* @param lsaf_from - optional - The beginning of the date range for items deleted within a      *;
%*            specific time period. Values must be one of the formats described below. Date      *;
%*            values in formats other than the two described below will result in an execution   *;
%*            failure.                                                                           *;
%*            - SAS datetime18. format (e.g., 02NOV2013:09:33:22) -  an offset of zero hours     *;
%*              from GMT will be used.                                                           *;
%*            - SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).                         *;
%* @param lsaf_to - optional - The end of the date range for items deleted within a              *;
%*            specific time period. Values must be one of the formats described below. Date      *;
%*            values in formats other than the two described below will result in an execution   *;
%*            failure.                                                                           *;
%*            - SAS datetime18. format (e.g., 02NOV2013:09:33:22) -  an offset of zero hours     *;
%*              from GMT will be used.                                                           *;
%*            - SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).                         *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the items specified as libref.dataset. The default value is           *;
%*            WORK.LSAFGETRBITEMSBYQUERY. The data set contains a row for each item in the list  *;
%*            and columns with the following names.  The data set will be sorted by path.        *;
%*            - name: The name of the item.                                                      *;
%*            - path: The path of the item.                                                      *;
%*            - objectType: Describes the type of the item (e.g. folder, data set, etc.)         *;
%*            - identifier: The unique identifier of the Repository object.                      *;
%*            - size: The size of the object in bytes (will be 0 for containers).                *;
%*            - deletedBy: The user ID that deleted the object from the Repository.              *;
%*            - deletedOn: The full date and time when the object was deleted from the           *;
%*                  Repository represented as a String.                                          *;
%*            - dateDeleted: The date when the object was deleted as a SAS Datetime format.      *;
%*            - lastModifiedBy: The user ID that last modified the object.                       *;
%*            - lastModifiedOn: The date when the object was last modified.  This value is       *;
%*                  represented as a String.                                                     *;
%*            - dateLastModified: The date when the object was last modified as a SAS Datetime   *;
%*                  format.                                                                      *;

%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2017-01-24  original coding.                                                         *;
%*                                                                                               *;

%macro lsaf_getrbitemsbyquery(
  lsaf_path=,
  lsaf_name=,
  lsaf_userid=,
  lsaf_from=,
  lsaf_to=,
  sas_dsname=work.lsafGetRBItemsByQuery
  ) /des='Get a list of the items in the recycle bin based on given criteria.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path name objectType identifier size deleted deletedBy dateDeleted lastModified lastModifiedBy dateLastModified
                         sortPath);
    attrib
           returnCode         length=8
           message            length=$200
           logMsgPrefix       length=$70
           logMsg             length=$300
           rowCount           length=8

           identifier         length=$512                     label="Unique Identifier"
           path               length=$2048                    label="Object Path"
           name               length=$256                     label="Object Name"
           objectType         length=$32                      label="Object Type"

           size               length=8                        label="Size In Bytes"

           deletedBy          length=$64                      label="Userid that deleted the object"
           deleted            length=$64                      label="Date the object was deleted"
           dateDeleted        length=8      format=datetime.  label="Date the object was deleted (SAS DateTime Format)"

           lastModifiedBy     length=$64                      label="Userid that last modified the object content"
           lastModified       length=$64                      label="Date the object's content was last modified"
           dateLastModified   length=8      format=datetime.  label="Date the object's content was last modified (SAS DateTime Format)"

           tempDateDeleted    length=$64                      label="USED FOR FORMATTING - will be dropped"
           tempDateModified   length=$64                      label="USED FOR FORMATTING - will be dropped"
           sortpath           length=$2048                    label="NEEDED FOR SORTING - will be dropped"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RecycleBinService") ;
    declare javaobj results("com/sas/hls/macro/results/SasRecycleBinResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getRecycleBinItemsByQuery", "%bquote(&lsaf_path)", "%bquote(&lsaf_name)", "%bquote(&lsaf_userid)",
                                                          "%bquote(&lsaf_from)", "%bquote(&lsaf_to)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve list of items in the recycle bin." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Returned the list of items in the recycle bin.";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        sortPath = kstrip( kupcase( ktranslate(path,'1','_') ) );

        results.callStringMethod("getName", row, name);
        results.callStringMethod("getObjectType", row, objectType);
        results.callStringMethod("getIdentifier", row, identifier);
        results.callLongMethod("getItemSize", row, size);

        results.callStringMethod("getDeletedBy", row, deletedBy);
        results.callStringMethod("getDeleted", row, deleted);
        results.callStringMethod("getDeletedSasFormat", row, tempDateDeleted);
        dateDeleted=input(tempDateDeleted, datetime.);

        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, tempDateModified);
        dateLastModified=input(tempDateModified, datetime.);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
        put logMsgPrefix " No recycle bin items were found matching the given criteria.";
      else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %if (&_lsafRowCount_ > 0) %then
  %do;
     proc sort data=&sas_dsname
               out=&sas_dsname (drop=sortpath);
         by sortpath;
     run;
  %end;
  %else
  %do;
     proc sql;
       alter table &sas_dsname
          drop sortpath;
     quit;
  %end;
  options &notesOption;

%mend lsaf_getrbitemsbyquery ;
