%* lsaf_getrelativepath                                                                          *;
%*                                                                                               *;
%* Returns a relative path for a file relative to the given base path. Neither path is validated *;
%* for existence in SAS Life Science Analytics Framework or for valid characters. Logging in to  *;
%* SAS Life Science Analytics Framework is not required to perform this operation.               *;
%*                                                                                               *;
%* The macro variable _lsafRelativePath_ will be set with the relative path or blank if the      *;
%* operation fails.                                                                              *;
%*                                                                                               *;
%* Syntax                                                                                        *;
%* %LSAF_GETRELATIVEPATH(LSAF_BASE_PATH=lsaf-base-path, LSAF_ABSOLUTE_PATH=lsaf-absolute-path);  *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafRelativePath_ the generated relative path.                                       *;
%*                                                                                               *;
%* @param lsaf_base_path - required - the base container location. The base path must be an      *;
%*            absolute path.                                                                     *;
%* @param lsaf_absolute_path - required - absolute path of the file to make relative to the base *;
%*            path.                                                                              *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-10-25  original coding                                                          *;
%*                                                                                               *;

%macro lsaf_getrelativepath(
  lsaf_base_path=,
  lsaf_absolute_path=
  ) /des='Get the path relative to the base path.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafRelativePath_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsg          length=$300
           relativePath    length=$2048
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/UtilityService") ;
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getRelativePath", "%bquote(&lsaf_base_path)", "%bquote(&lsaf_absolute_path)", results, relativePath);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot generate relative path for &lsaf_absolute_path. based on &lsaf_base_path." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The relative path was generated: " || kstrip(relativePath) ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " _lsafRelativePath_= " relativePath;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRelativePath_", kstrip(relativePath)) ;

  run;

%mend lsaf_getrelativepath;
