%* lsaf_getrolemembers                                                                              *;
%*                                                                                                  *;
%* Returns a SAS data set that contains the metadata for all of the members assigned to a role      *;
%* within the specified organization, project, or analysis.                                         *;
%*                                                                                                  *;
%* Syntax:                                                                                          *;
%* %LSAF_GETROLEMEMBERS(LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name,                              *;
%* LSAF_ROLE_CONTEXT=lsaf-role-context,  <, SAS_DSNAME=SAS-data-set>);                              *;
%*                                                                                                  *;
%* @macvar _lsafRC_  the return code.                                                               *;
%* @macvar _lsafMsg_ the return message.                                                            *;
%*                                                                                                  *;
%* @param lsaf_path - required - the path of the context object (organization, project, or          *;
%*            analysis) where the role is assigned.                                                 *;
%* @param lsaf_role - required - the name of the role.                                              *;
%* @param lsaf_role_context - required - specifies the context in which the role is defined as a    *;
%*            path (e.g., /SAS).                                                                    *;
%* @param sas_dsname - optional - the name of the output SAS data set that contains the metadata    *;
%*            for the members specified as libref.dataset. The default value is                     *;
%*            WORK.LSAFGETROLEMEMBERS. The data set contains a row for each member in the role and  *;
%*            columns with the following names. The data set will be sorted by member type and      *;
%*            principalId.                                                                          *;
%*            - contextPath: The path of the context object.                                        *;
%*            - type: The member type. Valid values are user or group.                              *;
%*            - principalId: The name of the member. Valid values are group name or user ID.        *;
%*            - descriptor: The description of the member. Valid values are group name or user      *;
%*                  display name.                                                                   *;
%*            - grpSrcCtxt: The context in which the group member was defined, if the member is     *;
%*                  a group.                                                                        *;
%*                                                                                                  *;
%* @macrotype LSAF                                                                                  *;
%* @since 1.2                                                                                       *;
%* @exposure external                                                                               *;
%*                                                                                                  *;
%* @history 2013-08-30  updates for standardization and coding conventions                          *;
%* @history 2016-08-26  rebrand as LSAF                                                             *;
%*                                                                                                  *;

%macro lsaf_getrolemembers(
  lsaf_path=,
  lsaf_role=,
  lsaf_role_context=,
  sas_dsname=work.lsafGetRoleMembers
  ) /des='Get a list of the members of a role in an LSAF context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local _lsafRowCount_;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let _lsafRowCount_=0;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath type principalId description grpSrcCtxt);
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib contextPath     length=$2048     label="Context Path" ;
    attrib type            length=$32       label="Principal Type";
    attrib principalId     length=$256      label="Principal Name";
    attrib description     length=$128      label="Principal Description";
    attrib grpSrcCtxt      length=$2048     label="Group Context Source Path";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RoleService");
    declare javaobj results("com/sas/hls/macro/results/SasPrincipalsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getMembers", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", "%bquote(&lsaf_role_context)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve members for role &lsaf_role. defined at &lsaf_role_context. for context path &lsaf_path.." ;
    end;
      else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Members were returned for role &lsaf_role. defined at &lsaf_role_context. for context path &lsaf_path.." ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        contextPath="&lsaf_path";
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getName", row, principalId);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getContextPath", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    if (rowCount=0) then
      put logMsgPrefix " No members were found.";
    else
      put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by principal id            *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by type principalId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getrolemembers;
