%* lsaf_getroles                                                                                  *;
%*                                                                                                *;
%* Returns a SAS data set that contains the metadata for all of the roles defined and, if         *;
%* specified, inherited within the specified organization, project, or analysis.                  *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETROLES(LSAF_PATH=lsaf-path <,SAS_DSNAME=SAS-data-set,                                  *;
%* LSAF_GETINHERITED=inheritance-flag>);                                                          *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the context object (organization, project, or        *;
%*            analysis) where the roles exist.                                                    *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the       *;
%*            metadata for the roles specified as libref.dataset. The default value is            *;
%*            WORK.LSAFGETROLES. The data set contains a row for each role in the context object  *;
%*            as well as columns with the following names.  The data set will be sorted by role   *;
%*            name.                                                                               *;
%*            - path: The path of the context object.                                             *;
%*            - name: The name of the role.                                                       *;
%*            - description: The role description.                                                *;
%*            - roleSrcCtxt: The context in which the of role was defined.                        *;
%*            - isInherited: A flag indicating whether or not the role is inherited. The valid    *;
%*                  values are 0 and 1.  This variable is not included if LSAF_GETINHERITED is    *;
%*                  not valid or set to 0.                                                        *;
%* @param lsaf_getinherited - optional -  indicates whether or not to include inherited roles.    *;
%*            The valid values are 0 and 1 to exclude or include inherited roles respectively.    *;
%*            The default is 1.                                                                   *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%*                                                                                                *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;

%macro lsaf_getroles(
  lsaf_path=,
  sas_dsname=work.lsafGetRoles,
  lsaf_getInherited=1
  ) /des='Get a list of roles in a SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=contextPath name description roleSrcCtxt
    %if (&lsaf_getInherited eq %str(1)) %then
    %do;
        isInherited
    %end;
  );
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib contextPath     length=$2048     label="Context Path" ;
    attrib name            length=$256      label="Role Name";
    attrib description     length=$128      label="Role Description";
    attrib roleSrcCtxt     length=$2048     label="Role Context Source Path";

    %if (&lsaf_getInherited eq %str(1)) %then
    %do;
      attrib isInherited   length=8         label="Is role inherited in current context: 0 or 1";
    %end;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RoleService") ;
    declare javaobj results("com/sas/hls/macro/results/SasRolesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getRoles", "%bquote(&lsaf_path)", "%bquote(&lsaf_getInherited)", results) ;

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve roles for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Roles were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        contextPath="&lsaf_path";
        results.callStringMethod("getContextPath", row, roleSrcCtxt);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        %if (&lsaf_getInherited eq %str(1)) %then
        %do;
          results.callIntMethod("isInherited", row, isInherited);
        %end;
          output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No roles were found.";
     else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort by name                    *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;


%mend lsaf_getroles;
