%* lsaf_getsubmissionstatus                                                                      *;
%*                                                                                               *;
%* Determines the status of a SAS Life Science Analytics Framework repository or workspace job   *;
%* submission.                                                                                   *;
%*                                                                                               *;
%* The SAS macro variable _lsafJobSubmissionStatus_ is set to the status value. The values are   *;
%* NOT_SET, NOT_STARTED, STARTED, RUNNING_TASKS, POST_EXECUTION_PUBLISHING, COMPLETED_SUCCESSFUL,*;
%* COMPLETED_SASERRORS, COMPLETED_CHECKIN_ERROR, COMPLETED_SASERRORS_CHECKIN_ERROR,              *;
%* COMPLETED_FAILED, COMPLETED_CANCELED, COMPLETED_SASWARNINGS, and                              *;
%* COMPLETED_SASWARNINGS_CHECKIN_ERROR.                                                          *;
%* If the macro runs without error, _lsafRC_ will be set to 0, the _lsafJobSubmiisionStatus_     *;
%* macro variable will be set to the appropriate value, and a job submission status message will *;
%* be printed to the log. If the job completed with warnings or errors, the job submission       *;
%* message will show in the log as a warning or an error, as appropriate.                        *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETSUBMISSIONSTATUS(LSAF_JOBSUBMISSION_ID=job-submission-id)                            *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafJobSubmissionStatus_ indicates the job submission status.                        *;
%*                                                                                               *;
%* @param lsaf_jobsubmission_id - required - the submission identifier returned from the         *;
%*         lsaf_submitjob macro.                                                                 *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-04-07  initial code                                                             *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%* @history 2016-11-02  added job submission message, updated doc                                *;
%*                                                                                               *;

%macro lsaf_getsubmissionstatus(
  lsaf_jobsubmission_id=
  ) /des='What is the submission status of SAS Life Science Analytics Framework job?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafJobSubmissionStatus_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsg          length=$300
           status          length=$25
           msgLevel        length=$25
           jobMsg          length=$300
    ;

    status="";
    msgLevel="";
    jobMsg="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/JobExecutionService");
    declare javaobj results("com/sas/hls/macro/results/SasJobSubmissionResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getJobSubmissionStatus", "%bquote(&lsaf_jobsubmission_id)", results);
    results.callStringMethod("getStatus", status);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine submission status for identifier: &lsaf_jobsubmission_id.." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Returned submission status: &lsaf_jobsubmission_id.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafJobSubmissionStatus_= " status;
    if (returnCode EQ 0) then
    do;
       results.callStringMethod("getStatusMessage", jobMsg);
       if (jobMsg NE "") then
       do;
          results.callStringMethod("getLogMsgLevel", msgLevel);
          msgText= strip(msgLevel) || ": &_LSAF_PRODUCT_NAME_ Macro: * " ||  strip(jobMsg);
          put msgText;
       end;
    end;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafJobSubmissionStatus_", strip(status) ) ;

  run;

%mend lsaf_getsubmissionstatus;
