%* lsaf_getsubscriptions                                                                         *;
%*                                                                                               *;
%* Returns a SAS data set containing metadata for all subscriptions for the current user.        *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETSUBSCRIPTIONS(<SAS_DSNAME=SAS-data-set>);                                            *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the subscriptions specified as libref.dataset. The default value is   *;
%*            WORK.LSAFGETSUBSCRIPTIONS. The data set contains a row for each subscription       *;
%*            with the following names.  The data set will be sorted by event, location, and     *;
%*            principalId.                                                                       *;
%*            - event: The name of the event subscription.                                       *;
%*            - location: The path of the object to which a subscription applies.                *;
%*            - type:  The subscriber type.  Values are USER and GROUP.                          *;
%*            - cascade: Indicates if the subscription is cascaded to child items.  Values are   *;
%*                  0 and 1.                                                                     *;
%*            - principalId: The id of the subscriber.  The returned values will be either a     *;
%*                  group name or the user ID of the current user.                               *;
%*            - grpSrcCtxt: The context path in which the group was defined if the subscription  *;
%*                  is a for group.                                                              *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-03-11  initial coding                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_getsubscriptions(
  sas_dsname=work.lsafGetSubscriptions
  ) /des='Get a list of subscriptions for the current user';


  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (
          label="Created by &SYSMACRONAME"
          keep=event location cascade type principalId grpSrcCtxt
          );
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

   attrib tempCascade     length=$1;

    attrib event           length=$64       label="Event Name";
    attrib location        length=$2048     label="Location";
    attrib cascade         length=8         label="Cascade";
    attrib type            length=$32       label="Principal Type";
    attrib principalId     length=$256      label="Principal Name";
    attrib grpSrcCtxt      length=$2048     label="Group Context Source Path";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/SubscriptionService");
    declare javaobj results("com/sas/hls/macro/results/SasSubscriptionsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getSubscriptionsForCurrentUser", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve subscriptions" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Subscriptions were returned for the current user" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getEvent", row, event);
        results.callStringMethod("getLocation", row, location);
       results.callStringMethod("getCascade", row, tempCascade);
      cascade = input(tempCascade, 8.);
        results.callStringMethod("getType", row, type);
        results.callStringMethod("getPrincipalId", row, principalId);
        results.callStringMethod("getGrpSrcCtxt", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No subscriptions were found.";
      else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
   %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by event location principalId;
    run;
    options &notesOption;
  %end;

%mend lsaf_getsubscriptions;
