%* lsaf_getsyncinfo                                                                                *;
%*                                                                                                 *;
%* Returns a SAS data set containing synchronization information for the specified file in the     *;
%* SAS Life Science Analytics Framework repository.                                                *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETSYNCINFO(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                             *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the job.                                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            synchronization information for the file specified as libref.dataset. The default    *;
%*            value is WORK.LSAFGETSYNCINFO. The data set contains a single row for each           *;
%*            synchronization property associated with the file. The output data set contains the  *;
%*            columns - name, value, and description. The NAME column contains the name of the     *;
%*            property, the DESCRIPTION column contains a brief explanation of the property, and   *;
%*            the VALUE column contains the value of the property. Listed below are the NAME and   *;
%*            DESCRIPTION column values:                                                           *;
%*                                                                                                 *;
%*            NAME                           DESCRIPTION                                           *;
%*		      filePath                       File path                                             *;
%*		      fileCheckOutStatus:            The file check out status. Values are                 *;
%*                                           NOT_CHECKED_OUT, CHECKED_OUT, and MARKED_FOR_ADD      *;
%*		      fileStatus:                    File status. Values are NON_EXISTENT, WORKSPACE_ONLY, *;
%*                                           REPOSITORY_ONLY, and WORKSPACE_AND_REPOSITORY         *;
%*		      syncFileStatus:                File synchronization status. Values are NOT_SYNCED,   *;
%*                                           IN_SYNC, and OUT_OF_SYNC                              *;
%*		      repoFileCheckedOut:            File checked out. Values are 1 for yes, and 0 for no  *;
%*		      repoFileCheckedOutUser:        The user that has the repository file checked out.    *;
%*		      repoLastModifiedDate:          Date the repository file was last modified            *;
%*		      repoLastModifiedSASDate        SAS date the repository file was last modified        *;
%*		      repoFileVersion:               File version                                          *;
%*            repoFileSize:                  File size                                             *;
%*            repoFileSizeFormatted:         Formatted file size                                   *;
%*		      syncFileVersion:               The version of the repository file that was           *;
%*                                           synchronized to the Workspace                         *;
%*		      syncFileRepoSize:              The file size of the repository file that was         *;
%*                                           synchronized to the workspace                         *;
%*		      syncFileRepoSizeFormatted:     The formatted file size of the repository file that   *;
%*                                           was synchronized to the workspace                     *;
%*		      syncRepoLastModifiedDate:      The date of the repository file version that was      *;
%*                                           synchronized to the workspace was last modified       *;
%*		      syncRepoLastModifiedSASDate:   The SAS date of the repository file version that was  *;
%*                                           synchronized to the workspace was last modified       *;
%*		      syncDate:                      The file synchronization date                         *;
%*		      syncSASDate:                   The file synchronization SAS date                     *;
%*		      workspaceFileSize:             The file size of the workspace file                   *;
%*		      workspaceFileSizeFormatted:    The formatted file size of the workspace file         *;
%*		      workspaceLastModifiedDate:     Date the workspace file was last modified             *;
%*		      workspaceLastModifiedSASDate:  SAS date the workspace file was last modified         *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-10-28  initial coding.                                                            *;
%*                                                                                                 *;

%macro lsaf_getsyncinfo(
   lsaf_path=,
   sas_dsname=work.lsafGetSyncInfo
   ) /des='Get the synchronization information for a file.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=name value description
   );

   attrib
          rowCount        length=8
          returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsg          length=$300

		  name            length=$32
		  value			  length=$2048
		  description     length=$200
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/hls/macro/service/RepositoryService") ;
   declare javaobj results("com/sas/hls/macro/results/SasSyncPropertiesResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getSynchInfo", "%bquote(&lsaf_path)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   if (returnCode ne 0) then
   do;
     logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
     logMsg=" Cannot retrieve synchronization information for file &lsaf_path" ;
   end;
   else
   do;
     logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
     logMsg=" Returned a list of workpspace objects in %bquote(&lsaf_path)";
     results.callIntMethod("getSize", rowCount);
     do row=0 to rowCount - 1;
       results.callStringMethod("getPropertyName", row, name);
       results.callStringMethod("getPropertyValue", row, value);
       results.callStringMethod("getPropertyDescription", row, description);
       output;
     end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   put logMsgPrefix " " logMsg;
   if (rowCount=0) then
     put logMsgPrefix " No synchronization information was found.";
   else
     put logMsgPrefix " Synchronization information written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getsyncinfo;
