%* lsaf_getsyncstatus                                                                             *;
%*                                                                                                *;
%* Determines if a file in the SAS Life Science Analytics Framework repository is synchronized,   *;
%* out of synchronization, or has never been synchronized with the workspace.                     *;
%*                                                                                                *;
%* The SAS macro variable _lsafSyncStatus_ is set to the status value. The values are NOT_SYNCED, *;
%* IN_SYNC, and OUT_OF_SYNC.  If the file does not exist in either the workspace or the           *;
%* repository, the status is reported as NOT_SYNCED. If the object is a container, an error will  *;
%* be reported.                                                                                   *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETSYNCSTATUS(LSAF_PATH=lsaf-path);                                                      *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%* @macvar _lsafSyncStatus_ indicates the syncronization status.                                  *;
%*                                                                                                *;
%* @param lsaf_path - required -  the path of the file.                                           *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2014-04-07  initial code                                                              *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%*                                                                                                *;

%macro lsaf_getsyncstatus(
  lsaf_path=
  ) /des='What is the synchronization status of SAS Life Science Analytics Framework file?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafSyncStatus_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib status          length=$25;

    status="";

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticStringMethod("getSyncStatus", "%bquote(&lsaf_path)", results, status);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine synchronization status: &lsaf_path." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Returned synchronization status: &lsaf_path." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafSyncStatus_= " status;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafSyncStatus_", strip(status) ) ;

  run;

%mend lsaf_getsyncstatus ;
