%* lsaf_getusergroups                                                                            *;
%*                                                                                               *;
%* Returns a SAS data set containing metadata for each of the groups in which the specified      *;
%* user is a member.                                                                             *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETUSERGROUPS(LSAF_USERID=lsaf-userid <, SAS_DSNAME=SAS-data-set>);                     *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @param lsaf_userid - required - the user ID of the user for which groups will be retrieved.   *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the groups specified as libref.dataset. The default value is          *;
%*            WORK.LSAFGETUSERGROUPS. The data set contains a row for each group and columns     *;
%*            with the following names.  The data set will be sorted by group name and           *;
%*            group context path.                                                                *;
%*            - userId: The user ID of the user.                                                 *;
%*            - name: The name of the group.                                                     *;
%*            - description: The description of the group.                                       *;
%*            - grpSrcCtxt: The context path where the group was defined.                        *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2014-03-20  initial coding                                                           *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_getusergroups(
  lsaf_userid=,
  sas_dsname=work.lsafGetUserGroups
  ) /des='Get a list of the groups in which a user is a member.';


  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_ notesOption;

  %let _lsafRowCount_=0;

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data &sas_dsname (
          label="Created by &SYSMACRONAME"
          keep=userId name description grpSrcCtxt
          );
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib rowCount        length=8;

    attrib userId          length=$256      label="User Id";
    attrib name            length=$256      label="Group Name";
    attrib description     length=$128      label="Group Description";
    attrib grpSrcCtxt      length=$2048     label="Group Context Source Path";

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/GroupService");
    declare javaobj results("com/sas/hls/macro/results/SasPrincipalsResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getUserGroups", "%bquote(&lsaf_userid)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve groups for &lsaf_userid." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Operation to retrieve groups was successful for &lsaf_userid." ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        userId="&lsaf_userid";
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getContextPath", row, grpSrcCtxt);
        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
          put logMsgPrefix " No groups were found.";
      else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
   %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name grpSrcCtxt;
    run;
    options &notesOption;
  %end;

%mend lsaf_getusergroups;
