%* lsaf_getversions                                                                               *;
%*                                                                                                *;
%* Returns a SAS data set that contains the metadata for all of the versions of a file.           *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETVERSIONS(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                            *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required -the path of the object in the SAS Life Science Analytics          *;
%*            Framework repository.                                                               *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the       *;
%*            metadata for the versions specified as libref.dataset. The default value is         *;
%*            WORK.LSAFGETVERSIONS. The data set contains a row for each version of the object    *;
%*            and columns with the following names.  The most recent version will be the first    *;
%*            observation in the data set.                                                        *;
%*            - name: The name of the object.                                                     *;
%*            - path: The path of the object.                                                     *;
%*            - objectType: The type of file (i.e., data set, program, etc.).                     *;
%*            - description: The description of the object.                                       *;
%*            - versionLabel: The label associated with this specific version.                    *;
%*            - isCheckedOut: A numeric flag that indicates whether the object is currently       *;
%*                  checked out (0=no, 1=yes).                                                    *;
%*            - isLocked:  A numeric flag that indicates whether the object is currently locked   *;
%*                  (0=no, 1=yes).                                                                *;
%*            - size: The size of the object in bytes (will be 0 for containers).                 *;
%*            - contentType: The content or MIME type of object.                                  *;
%*            - createdBy: The user ID that created the object.                                   *;
%*            - created: The full date and time the object was created represented as a string.   *;
%*            - dateCreated: The date the object was created as a SAS Datetime format.            *;
%*            - lastModifiedBy: The user ID that last modified the object.                        *;
%*            - lastModified: The date the object was last modified represented as a string.      *;
%*            - dateLastModified: The date the object was last modified as a SAS Datetime format. *;
%*            - propertiesLastModifiedBy: The user ID that last modified the properties for the   *;
%*                  object.                                                                       *;
%*            - propertiesLastModified: The date when the object properties were last modified.   *;
%*                  This value is represented as a String.                                        *;
%*            - datePropertiesLastModified: The date when the object properties were last         *;
%*                  modified as a SAS Datetime format.                                            *;
%*            - isSigned: Indicates if the object has electronic signatures attached to it.       *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.2                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2013-08-30  updates for standardization and coding conventions                        *;
%* @history 2014-05-07  updated documentation and fixed data set processing.                      *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%*                                                                                                *;

%macro lsaf_getversions(
  lsaf_path=,
  sas_dsname=work.lsafGetVersions
  ) /des='Get a list of all the versions of an object in the SAS Life Science Analytics Framework in the repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path name description objectType
                         description versionLabel isCheckedOut isLocked isSigned size contentType
                         createdBy created dateCreated
                         lastModifiedBy lastModified dateLastModified
                         propertiesLastModifiedBy propertiesLastModified datePropertiesLastModified
                         isSigned uppath);

    attrib returnCode                length=8
           message                   length=$200
           logMsgPrefix              length=$70
           logMsg                    length=$300
           rowCount                  length=8

           path                      length=$2048    label="Object Path"
           name                      length=$256     label="File name"
           objectType                length=$32      label="Object Type"

           description               length=$512     label="Description"
           versionLabel              length=$32      label="Version Number"
           isCheckedOut              length=8        label="Is File Checked Out"
           isLocked                  length=8        label="Is File Locked"
           isSigned                  length=8        label="Is File Signed"
           size                      length=8        label="Size in Bytes"
           contentType               length=$32      label="Content Type"

           createdBy                 length=$64      label="Userid that created the object"
           created                   length=$64      label="Object Creation Date"
           dateCreated                               label="Date the object was created (SAS DateTime Format)"

           lastModifiedBy            length=$64      label="Userid that last modified the objects content"
           lastModified              length=$64      label="Last Content Modification Date"
           dateLastModified                          label="Date the object's content was last modified (SAS DateTime Format)"

           propertiesLastModifiedBy  length=$64      label="Userid that last modified the object properties"
           propertiesLastModified    length=$64      label="Last Properties Modification Date"
           datePropertiesLastModified               label="Date the object's properties were last modified (SAS DateTime Format)"

           createdTempSAS            length=$64
           lastModifiedTempSAS       length=$64
           propertiesLastModifiedTempSAS  length=$64

           uppath                    length=$2048    label="FOR SORTING - WILL BE DROPPED"
    ;

    format dateCreated
           dateLastModified
           datePropertiesLastModified datetime.
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService") ;
    declare javaobj results("com/sas/hls/macro/results/SasResourcesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getVersions", "%bquote(&lsaf_path)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve list of versions for &lsaf_path" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" List of versions were returned for &lsaf_path" ;
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callStringMethod("getVersionLabel", row, versionLabel);
        results.callIntMethod("isCheckedOut", row, isCheckedOut);
        results.callIntMethod("isLocked", row, isLocked);
        results.callIntMethod("isSigned", row, isSigned);
        results.callLongMethod("getSize", row, size);
        results.callStringMethod("getContentType", row, contentType);
        results.callStringMethod("getObjectType", row, objectType);
        results.callStringMethod("getCreated", row, created);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getPropertiesLastModified", row, propertiesLastModified);
        results.callStringMethod("getPropertiesLastModifiedBy", row, propertiesLastModifiedBy);

        results.callStringMethod("getCreatedSasFormat", row, createdTempSas);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);
        results.callStringMethod("getPropertiesLastModifiedSasFormat", row, propertiesLastModifiedTempSAS);

        dateCreated=input(createdTempSAS, datetime.);
        dateLastModified=input(lastModifiedTempSas, datetime.);
        datePropertiesLastModified=input(propertiesLastModifiedTempSAS, datetime.);

        uppath=upcase(path) ;
        uppath=translate(uppath,'1','_');

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
        put logMsgPrefix " No versions were found.";
      else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

   run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by uppath;
    run;
  %end;
  %***********************************;
  %* Cleanup                         *;
  %***********************************;
  proc sql;
     alter table &sas_dsname
        drop uppath;
  quit;

  options &notesOption;

%mend lsaf_getversions ;
