%* lsaf_getworkspacechildren                                                                     *;
%*                                                                                               *;
%* Returns a SAS data set that contains the metadata for objects within a folder in the SAS      *;
%* Life Science Analytics Framework workspace.                                                   *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GETWORKSPACECHILDREN(LSAF_PATH=lsaf-path <, LSAF_RECURSIVE=recursion-level,             *;
%* SAS_DSNAME=SAS-data-set>);                                                                    *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the folder object in the SAS Life Science           *;
%*            Analytics Framework workspace.                                                     *;
%*                                                                                               *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the objects specified as libref.dataset. The default value is         *;
%*            WORK.LSAFGETWORKSPACECHILDREN. The data set contains a row for each object in the  *;
%*            list and columns with the following names.  The data set will be sorted by path.   *;
%*            - name: The name of the object.                                                    *;
%*            - path: The path of the object.                                                    *;
%*            - objectType: Describes the type of the object.                                    *;
%*            - isFolder: A numeric flag that indicates whether the object is a folder           *;
%*                  object (0=no, 1=yes).                                                        *;
%*            - lastModified: The date when the object was last modified.  This value is         *;
%*                  represented as a String.                                                     *;
%*            - dateLastModified: The date when the object was last modified as a SAS Datetime   *;
%*                  format.                                                                      *;
%*                                                                                               *;
%* @param lsaf_recursive - optional - the level of recursion.                                    *;
%*            Valid values are 0, 1, 2, or 99 as described below. The default value is 0.        *;
%*                                                                                               *;
%*             0 - Returns the folder only.                                                      *;
%*             1 - Returns the folder and its immediate children.                                *;
%*             2 - Returns the folder, its immediate children, and all objects within one        *;
%*                 level of the folders subfolders.                                              *;
%*            99 - Returns the folder and all of the sub folders and objects                     *;
%*                 (traverses the entire tree).                                                  *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-11-09  initial coding                                                           *;
%*                                                                                               *;

%macro lsaf_getworkspacechildren(
  lsaf_path=,
  sas_dsname=work.lsafGetWorkspaceChildren,
  lsaf_recursive=0
  ) /des='Get a list of the objects in a folder in SAS Life Science Analytics Framework workspace.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path name objectType isFolder
                         lastModified dateLastModified
                         uppath);

    attrib returnCode          length=8
           message             length=$200
           logMsgPrefix        length=$70
           logMsg              length=$300
           rowCount            length=8

           path                length=$2048    label="Object Path"
           name                length=$256     label="Object Name"
           objectType          length=$32      label="Object Type"
           isFolder            length=8        label="Is Object A Container?"

           lastModified        length=$64      label="Last Content Modification Date"
           dateLastModified                    label="Date the object's content was last modified (SAS DateTime Format)"

           lastModifiedTempSAS length=$64      label="TEMPORARY - WILL BE DROPPED"
           uppath              length=$2048    label="TEMPORARY - WILL BE DROPPED"
    ;

    format dateLastModified datetime.;
    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/WorkspaceService") ;
    declare javaobj results("com/sas/hls/macro/results/SasWorkspaceResourcesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getChildren", "%bquote(&lsaf_path)", "%bquote(&lsaf_recursive)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve list of objects from %bquote(&lsaf_path)" ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Returned a list of workpsace objects in %bquote(&lsaf_path)";
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
        results.callStringMethod("getPath", row, path);
        results.callStringMethod("getName", row, name);
        results.callIntMethod("isFolder", row, isFolder);
        results.callStringMethod("getObjectType", row, objectType);
        results.callStringMethod("getLastModified", row, lastModified);
        results.callStringMethod("getLastModifiedSasFormat", row, lastModifiedTempSas);

        dateLastModified=input(lastModifiedTempSas, datetime.);

        uppath=upcase(path) ;
        uppath=translate(uppath,'1','_');

        output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
        put logMsgPrefix " No children were found.";
      else
         put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " lsaf_recursive= &lsaf_recursive" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);

  run;

  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    proc sort data=&sas_dsname;
      by uppath;
    run;
  %end;
  %***********************************;
  %* Cleanup                         *;
  %***********************************;
  proc sql;
     alter table &sas_dsname
        drop uppath;
  quit;

  options &notesOption;

%mend lsaf_getworkspacechildren ;
