%* lsaf_getworkspacejobinfo                                                                       *;
%*                                                                                                *;
%* Returns a SAS data set that contains the information for a job in the SAS Life Science         *;
%* Analytics Framework workspace. This is the information displayed on the General tab of the     *;
%* job editor, as well as the information concerning the versioning of new outputs.               *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_GETWORKSPACEJOBINFO(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                    *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the job in the SAS Life Science Analytics Framework  *;
%*            workspace.                                                                          *;
%* @param sas_dsname - optional - the name of the output SAS data set that contains the metadata  *;
%*            for the job specified as libref.dataset. The default value is                       *;
%*            WORK.LSAFGETWORKSPACEJOBINFO. The data set will contain variables representing each *;
%*            element of metadata for the object.  The data set will be sorted by the NAME        *;
%*            variable.  The following metadata will be represented in the data set.              *;
%*            - path: The path of the job.                                                        *;
%*            - versionLabel: Versions are not relevant in the workspace. This variable is        *;
%*                 included for consistency with the data set returned by the repository macro    *;
%*                 lsaf_getjobinfo. The value from this macro call is WORKSPACE.                  *;
%*            - name: The name of the property or attribute.                                      *;
%*            - value: The value of the property or attribute.                                    *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-10-04  initial coding.                                                           *;
%*                                                                                                *;

%macro lsaf_getworkspacejobinfo(
  lsaf_path=,
  sas_dsname=work.lsafGetWorkspaceJobInfo
  ) /des='Get the metadata for SAS Life Science Analytics Framework workspace job.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %local _lsafRowCount_
         notesOption
  ;

  %let _lsafRowCount_=0;

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data &sas_dsname (label="Created by &SYSMACRONAME"
                    keep=path versionLabel name value);
    attrib
           returnCode      length=8
           message         length=$200
           logMsgPrefix    length=$70
           logMsg          length=$300

           path            length=$2048    label="Job Path"
           versionLabel    length=$32      label="Job Version Number"
           name            length=$256     label="Property Name"
           value           length=$2048    label="Property Value"
    ;

    rowCount=0;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/JobService") ;
    declare javaobj results("com/sas/hls/macro/results/SasJobPropertiesResult");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getWorkspaceJobInfo", "%bquote(&lsaf_path)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Could not retrieve job information." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Retrieved job information.";
      results.callStringMethod("getJobPath", path);
      results.callStringMethod("getJobVersion", versionLabel);
      results.callIntMethod("getSize", rowcount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPropertyName", row, name);
         results.callStringMethod("getPropertyValue", row, value);
         output;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
      if (rowCount=0) then
        put logMsgPrefix " The job information was not found.";
      else
         put logMsgPrefix  rowCount " records written to data set &sas_dsname";
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRowCount_", rowCount);
  run;

  %***********************************;
  %* Sort the data set               *;
  %***********************************;
  %if (&_lsafRowCount_ > 0) %then
  %do;
    %let notesOption=%sysfunc(getoption(notes));
    options nonotes;
    proc sort data=&sas_dsname;
      by name;
    run;
    options &notesOption;
  %end;

%mend lsaf_getworkspacejobinfo ;
