%* lsaf_getworkspacejoboutputs                                                                     *;
%*                                                                                                 *;
%* Returns a SAS data set containing metadata for the outputs for the specified job in the SAS     *;
%* Life Science Analytics Framework workspace.                                                     *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_GETJOBOUTPUTS(LSAF_PATH=lsaf-path <, SAS_DSNAME=SAS-data-set>);                           *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the job.                                              *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the        *;
%*            metadata for the outputs specified as libref.dataset. The default value is           *;
%*            WORK.LSAFGETWORKSPACEJOBOUTPUTS. The data set contains a row for each job output and *;
%*            columns with the following names. The data set will retain the order of the outputs. *;
%*            - jobPath: The path of the job.                                                      *;
%*            - jobVersion: Versions are not relevant in the workspace. This variable is           *;
%*                 included for consistency with the data set returned by the repository macro     *;
%*                 lsaf_getjoboutputs. The value from this macro call is WORKSPACE.                 *;
%*            - outputPath: The path of the output location.                                       *;
%*            - outputIncludesSubFolders: A flag indicating whether or not the subfolders of an    *;
%*              output container are to be included during job execution.                          *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-10-04  initial coding.                                                            *;
%*                                                                                                 *;

%macro lsaf_getworkspacejoboutputs(
   lsaf_path=,
   sas_dsname=work.lsafGetWorkspaceJobOutputs
   ) /des='Get the list of outputs for SAS Life Science Analytics Framework workspace job.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobPath jobVersion outputPath outputIncludesSubFolders
   );

   attrib
          returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsg          length=$300
          rowCount        length=8

          jobPath         length=$2048     label="Job Path"
          jobVersion      length=$32       label="Job Version"

          outputPath        length=$2048     label="Output Path"
          outputIncludesSubFolders length=8  label="Include Output Container SubFolders"

          includeSubFolders length=$5       label="FOR INTERNAL PROCESSING ONLY.  WILL BE DROPPED."
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/hls/macro/service/JobService") ;
   declare javaobj results("com/sas/hls/macro/results/SasJobResourceSpecificationResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getWorkspaceJobOutputs", "%bquote(&lsaf_path)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   if (returnCode ne 0) then
   do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot retrieve outputs for job &lsaf_path." ;
   end;
   else
   do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Outputs were returned for job &lsaf_path" ;
      results.callStringMethod("getJobPath", jobPath);
      results.callStringMethod("getJobVersion", jobVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPath", row, outputPath);
         results.callStringMethod("getIncludeSubFolders", row, includeSubFolders);
         outputIncludesSubFolders=input(includeSubFolders,best.);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   put logMsgPrefix " " logMsg;
   if (rowCount=0) then
      put logMsgPrefix " No outputs were found.";
   else
      put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getworkspacejoboutputs;
