%* lsaf_groupexists                                                                              *;
%*                                                                                               *;
%* Determines whether a group is defined for the specified context.                              *;
%*                                                                                               *;
%* The macro variable _lsafGroupExists_ will be set to 1 if the group exists, 0 if it does not.  *;
%* In the event of an incorrect call to the macro which results in an error, the return value of *;
%* _sddGroupExists_ will be set to -1.                                                           *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_GROUPEXISTS (LSAF_PATH=lsaf-path, LSAF_GROUP=lsaf- name);                               *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafGroupExists_ indicates if the group exists in the context.                       *;
%*                                                                                               *;
%* @param lsaf_path - required -  the path of the context object.                                *;
%* @param lsaf_group - required - the name of the group.                                         *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_groupexists(
  lsaf_path=,
  lsaf_group=
  ) /des='Is the group defined in the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafGroupExists_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/GroupService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("exists", "%bquote(&lsaf_path)", "%bquote(&lsaf_group)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the existence of group &lsaf_group in &lsaf_path.." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (exists) then
      do;
        logMsg=" The group &lsaf_group was found in &lsaf_path.";
      end;
      else
      do;
        logMsg=" The group &lsaf_group was not found in &lsaf_path.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafGroupExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafGroupExists_", exists) ;

  run;

%mend lsaf_groupexists ;
