%* lsaf_hascontextprivilege                                                                        *;
%*                                                                                                 *;
%* Determines whether a user or group has a specific role privilege in the specified               *;
%* organization, project, or analysis.                                                             *;
%*                                                                                                 *;
%* The SAS macro variable _lsafHasContextPrivilege_ is set to a value of 1 when the specified      *;
%* user or group has the privilege, 0 if it does not. In the event of an incorrect call to the     *;
%* macro which results in an error, the return value of _lsafHasContextPrivilege_ will be set to   *;
%* -1.                                                                                             *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_HASCONTEXTPRIVILEGE(LSAF_PATH=lsaf-path, LSAF_PRIVILEGE=lsaf-privilege-id,                *;
%* LSAF_MEMBER=lsaf-member-identifier <, LSAF_TYPE=lsaf-type,                                      *;
%* LSAF_GROUP_CONTEXT=lsaf-group-context>);                                                        *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%* @macvar _lsafHasContextPrivilege_ indicates if the user or group has the privilege in the       *;
%*            specified organization, project, or analysis.                                        *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) where the role is assigned.                                                *;
%* @param lsaf_privilege - required - the identifier of the privilege to check.                    *;
%* @param lsaf_member - required - the user ID or group name of the member.                        *;
%* @param lsaf_type - conditional - indicates the type of member. Valid values are USER and GROUP  *;
%*            (case-insensitive).  The default value is USER.                                      *;
%* @param lsaf_group_context - conditional - specifies the context in which the member group is    *;
%*            defined as a path (e.g., /SAS). The parameter is required if LSAF_TYPE=GROUP.        *;
%*            Otherwise, it will be ignored.                                                       *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.5                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2016-08-19  original code                                                              *;
%*                                                                                                 *;

%macro lsaf_hascontextprivilege(
  lsaf_path=,
  lsaf_privilege=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=
  ) /des='Does the user or group have the privilege in the specified context?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafHasContextPrivilege_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib hasPriv         length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RoleService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;

    srv.callStaticBooleanMethod("hasPrivilegeAtContext", "%bquote(&lsaf_path)", "%bquote(&lsaf_privilege)", "%bquote(&lsaf_type)",
                                "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", results, hasPriv);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if &lsaf_type &lsaf_member has the privilege &lsaf_privilege in &lsaf_path..";
      hasPriv=-1;
    end;
      else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (hasPriv) then
      do;
        logMsg=" The &lsaf_type &lsaf_member has the privilege &lsaf_privilege in &lsaf_path.." ;
      end;
        else
      do;
        logMsg=" The &lsaf_type &lsaf_member does not have the privilege &lsaf_privilege in &lsaf_path.." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafHasContextPrivilege_= " hasPriv;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafHasContextPrivilege_", hasPriv) ;

  run;

%mend lsaf_hascontextprivilege;
