%* lsaf_iscontainer                                                                                    *;
%*                                                                                                     *;
%* Determines whether a SAS Life Science Analytics Framework repository object is of type container.   *;
%*                                                                                                     *;
%* The SAS macro variable _lsafIsContainer_ is set to a value of 1 if the specified object is a        *;
%* container object, 0 if it is not.  In the event of an incorrect call to the macro which results in  *;
%* an error, the return value of _lsafIsContainer_ will be set to -1.                                  *;
%* Container objects include folders, projects, analyses, and the organization.                        *;
%*                                                                                                     *;
%* Syntax:                                                                                             *;
%* %LSAF_ISCONTAINER(LSAF_PATH=lsaf-path);                                                             *;
%*                                                                                                     *;
%* @macvar _lsafRC_  the return code.                                                                  *;
%* @macvar _lsafMsg_ the return message.                                                               *;
%* @macvar _lsafIsContainer_ indicates if the object is a container.                                   *;
%*                                                                                                     *;
%* @param lsaf_path - required - the path of the object in the SAS Life Science Analytics Framework    *;
%* repository.                                                                                         *;
%*                                                                                                     *;
%* @macrotype LSAF                                                                                     *;
%* @since 1.2                                                                                          *;
%* @exposure external                                                                                  *;
%*                                                                                                     *;
%* @history 2013-08-30  updates for standardization and coding conventions                             *;
%* @history 2016-08-26  rebrand as LSAF                                                                *;
%*                                                                                                     *;

%macro lsaf_iscontainer(
  lsaf_path=
  ) /des='Is the object a container?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsContainer_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib isContainer     length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isContainer", "%bquote(&lsaf_path)", results, isContainer);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the object is a container: &lsaf_path." ;
      isContainer=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (isContainer) then
      do;
        logMsg=" The object is a container: &lsaf_path.";
      end;
      else
      do;
        logMsg=" The object is not a container: &lsaf_path.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsContainer_= " isContainer;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsContainer_", isContainer);

  run;

%mend lsaf_iscontainer;
