%* lsaf_isgroupmember                                                                                    *;
%*                                                                                                       *;
%* Determines whether a user or group is an explicit member of the group in the specified organization,  *;
%* project, or analysis.                                                                                 *;
%*                                                                                                       *;
%* The SAS macro variable _lsafIsGroupMember_ is set to a value of 1 when the specified user or          *;
%* group is a member of the group, 0 if it is not. In the event of an incorrect call to the              *;
%* macro which results in an error, the return value of _lsafIsGroupMember_ will be set to -1.           *;
%*                                                                                                       *;
%* Syntax:                                                                                               *;
%* %LSAF_ISGROUPMEMBER(LSAF_PATH=lsaf-path, LSAF_GROUP=lsaf-group-name,                                  *;
%* LSAF_MEMBER=lsaf-member-identifier, <, LSAF_TYPE=lsaf-type, LSAF_GROUP_CONTEXT=lsaf-group-context,    *;
%*            LSAF_INCLUDEIMPLICIT=lsaf_include_implicit>);                                              *;
%*                                                                                                       *;
%* @macvar _lsafRC_  the return code.                                                                    *;
%* @macvar _lsafMsg_ the return message.                                                                 *;
%* @macvar _lsafIsGroupMember_ indicates if the user or group is a member of the specified group.        *;
%*                                                                                                       *;
%* @param lsaf_path - required -  the path of the context object (organization, project, or analysis)    *;
%*            where the group is assigned.                                                               *;
%* @param lsaf_group - required - the name of the group.                                                 *;
%* @param lsaf_member - required - the user ID or group name of the member.                              *;
%* @param lsaf_type - optional - indicates the type of member. Valid values are USER and GROUP           *;
%*            (case-insensitive).  The default value is USER.                                            *;
%* @param lsaf_group_context - conditional - specifies the context in which the member group is          *;
%*            defined as a path (e.g., /SAS). The parameter is required if LSAF_TYPE=GROUP.              *;
%*            Otherwise, it will be ignored.                                                             *;
%* @param lsaf_includeImplicit - optional -  indicates whether or not to include implied members, i.e.,  *;
%*            members of groups within groups. The valid values are 0 and 1 to exclude or include        *;
%*            implied memberships respectively. The default is 0.                                        *;
%*                                                                                                       *;
%* @macrotype LSAF                                                                                       *;
%* @since 1.2                                                                                            *;
%* @exposure external                                                                                    *;
%*                                                                                                       *;
%* @history 2013-08-30  updates for standardization and coding conventions                               *;
%* @history 2016-08-26  rebrand as LSAF                                                                  *;
%* @history 2016-01-18  added includeImplicit flag.                                                      *;
%*                                                                                                       *;

%macro lsaf_isgroupmember(
  lsaf_path=,
  lsaf_group=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=,
  lsaf_includeImplicit=0
  ) /des='Is the user or group a member of the group in the specified context?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsGroupMember_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %*  Start the datastep to call the java code  *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib isMember        length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/GroupService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("isMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_group)", "%bquote(&lsaf_type)", "%bquote(&lsaf_member)",
        "%bquote(&lsaf_group_context)", "%bquote(&lsaf_includeImplicit)", results, isMember);


    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if &lsaf_type &lsaf_member is a member of group &lsaf_group in &lsaf_path.";
      isMember=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (isMember) then
      do;
        logMsg=" The &lsaf_type &lsaf_member is a member of group &lsaf_group in &lsaf_path.";
      end;
      else
      do;
        logMsg=" The &lsaf_type &lsaf_member is not a member of group &lsaf_group in &lsaf_path.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " option lsaf_includeImplicit = &lsaf_includeImplicit";
    put logMsgPrefix " _lsafIsGroupMember_= " isMember;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsGroupMember_", isMember) ;

  run;

%mend lsaf_isgroupmember ;
