%* lsaf_isuserprivilege                                                                       *;
%*                                                                                            *;
%* Determines whether a global privilege is assigned to a user.                               *;
%*                                                                                            *;
%* The SAS macro variable _lsafIsUserPrivilege_ is set to a value of 1 when the privilege is  *;
%* assigned, 0 if it is not. In the event of an incorrect call to the macro which results in  *;
%* an error, the return value of _lsafIsUserPrivilege_ will be set to -1.                     *;
%*                                                                                            *;
%* Syntax:                                                                                    *;
%* %LSAF_ISUSERPRIVILEGE(LSAF_USERID=lsaf-userid, LSAF_PRIVILEGE=lsaf-privilege-id);          *;
%*                                                                                            *;
%* @macvar _lsafRC_  the return code.                                                         *;
%* @macvar _lsafMsg_ the return message.                                                      *;
%* @macvar _lsafIsUserPrivilege_ indicates if the privilege is assigned to the user.          *;
%*                                                                                            *;
%* @param lsaf_userid - required - the user ID to check.                                      *;
%* @param lsaf_privilege - required - the identifier of the global privilege to check.        *;
%*                                                                                            *;
%* @macrotype LSAF                                                                            *;
%* @since 1.3                                                                                 *;
%* @exposure external                                                                         *;
%*                                                                                            *;
%* @history 2013-11-01 initial coding                                                         *;
%* @history 2016-08-26  rebrand as LSAF                                                       *;
%*                                                                                            *;

%macro lsaf_isuserprivilege(
  lsaf_userid=,
  lsaf_privilege=
  ) /des='Is the global privilege assigned to the specified user?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafIsUserPrivilege_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib hasPrivilege    length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/PrivilegeService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("hasPrivilege", "%bquote(&lsaf_userid)", "%bquote(&lsaf_privilege)", results, hasPrivilege);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if &lsaf_privilege is assigned to the user &lsaf_userid..";
      hasPrivilege=-1;
    end;
      else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (hasPrivilege) then
      do;
        logMsg=" &lsaf_privilege is assigned to the user &lsaf_userid.." ;
      end;
      else
      do;
        logMsg=" &lsaf_privilege is not assigned to the user &lsaf_userid.." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafIsUserPrivilege_= " hasPrivilege;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafIsUserPrivilege_", hasPrivilege);

  run;

%mend lsaf_isuserprivilege ;
