%* lsaf_login                                                                                                   *;
%*                                                                                                              *;
%* Creates a connection to the SAS Life Science Analytics Framework instance.                                   *;
%*                                                                                                              *;
%* This macro is only required when running the macros from a PC SAS session. It must be called                 *;
%* before any other SAS Life Science Analytics Framework macro in order to establish your session.              *;
%*                                                                                                              *;
%* NOTE: This macro is only supported from a PC SAS session. Using this macro within an active                  *;
%* SAS Life Science Analytics Framework application session will result in a failure.                           *;
%*                                                                                                              *;
%* Syntax:                                                                                                      *;
%* %LSAF_LOGIN(LSAF_URL=url, LSAF_USERID=user-id , LSAF_PASSWORD=password,                                      *;
%*            <, PROXY_HOST=proxy-host, PROXY_USERID=proxy-userid, PROXY_PASSWORD=proxy-pw );                   *;
%*                                                                                                              *;
%* Example Code:                                                                                                *;
%*                                                                                                              *;
%* 1. The parameters passed to the login macro are often strings containing special characters                  *;
%*    which may cause problems when resolved by SAS.  You will probably want to wrap each of the                *;
%*    parameters in a SAS function call to prevent any issues:                                                  *;
%*    %lsaf_login(lsaf_url=%str(https://lsafinstance), lsaf_userid=%str(lsaf-user-id),                          *;
%*    lsaf_password=%str (lsafpassword ));                                                                      *;
%*                                                                                                              *;
%* 2. Certain special characters common in passwords (e.g. ampersands, percent signs) may                       *;
%*    may require quoting.                                                                                      *;
%*    %lsaf_login(lsaf_url=%str(https://lsafinstance), lsaf_userid=%str(lsaf-user-id),                          *;
%*    lsaf_password=%nrbquote(lsafpassword ));                                                                  *;
%*                                                                                                              *;
%* @macvar _lsafRC_  the return code.                                                                           *;
%* @macvar _lsafMsg_ the return message.                                                                        *;
%*                                                                                                              *;
%* @param lsaf_url - required - the URL of the SAS Life Science Analytics Framework instance to use             *;
%*             in the form https://lsafname.sas.com or http://lsafname.sas.com.                                 *;
%* @param lsaf_userid - required - the SAS Life Science Analytics Framework ID for the user account.            *;
%* @param lsaf_password - required - the SAS Life Science Analytics Framework password for the user account.    *;
%* @param proxy_host - conditional - the name of the host machine when the SAS Life Science Analytics           *;
%*             Framework system is accessed through a proxy server.                                             *;
%* @param proxy_port - conditional - the port number required for the proxy host machine.                       *;
%*             Required when a proxy_host is designated.                                                        *;
%* @param proxy_userid - conditional - the userid when the proxy machine requires authentication.               *;
%* @param proxy_password - conditional - the password when the proxy machine required                           *;
%*             authentication.                                                                                  *;
%*                                                                                                              *;
%* @macrotype LSAF                                                                                              *;
%* @since 1.1                                                                                                   *;
%* @exposure external                                                                                           *;
%*                                                                                                              *;
%* @history 2013-06-26  version 1.2 update - trusted session                                                    *;
%* @history 2013-08-30  updates for standardization and coding conventions                                      *;
%* @history 2014-03-04  updates for proxy                                                                       *;
%* @history 2016-08-26  rebrand as LSAF                                                                         *;
%*                                                                                                              *;

%macro lsaf_login(
  lsaf_url=,
  lsaf_userid=,
  lsaf_password=,
  proxy_host=,
  proxy_port=,
  proxy_userid=,
  proxy_password=
  ) /des='Initialize connection to SAS Life Science Analytics Framework.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/SessionService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("login", "%bquote(&lsaf_url)", "%bquote(&lsaf_userid)", "%bquote(&lsaf_password)",
         "%bquote(&proxy_host)", "%bquote(&proxy_port)", "%bquote(&proxy_userid)", "%bquote(&proxy_password)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg="Could not login to the &_LSAF_PRODUCT_NAME_ Server." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg="&_LSAF_PRODUCT_NAME_ Session initialized.";
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " url= &lsaf_url";
    put logMsgPrefix " userid= &lsaf_userid";
    if (klength(kstrip("&proxy_host")) > 0) then
    do;
       put logMsgPrefix " proxy host= &proxy_host";
       put logMsgPrefix " proxy port= &proxy_port";
       put logMsgPrefix " proxy_userid= &proxy_userid";
    end;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_login ;
