%* lsaf_objectexists                                                                        *;
%*                                                                                          *;
%* Determines whether a SAS Life Science Analytics Framework repository object exists.      *;
%*                                                                                          *;
%* The SAS macro variable _lsafObjectExists_ is set to a value of 1 when the object exits,  *;
%* it does not. In the event of an incorrect call to the macro which results in an error,   *;
%* the return value of _lsafObjectExists_ will be set to -1.                                *;
%*                                                                                          *;
%* Syntax:                                                                                  *;
%* %LSAF_OBJECTEXISTS(LSAF_PATH=lsaf-path);                                                 *;
%*                                                                                          *;
%* @macvar _lsafRC_  the return code.                                                       *;
%* @macvar _lsafMsg_ the return message.                                                    *;
%* @macvar _lsafObjectExists_ indicates if the object exists.                               *;
%*                                                                                          *;
%* @param lsaf_path - required -  the path of the object.                                   *;
%*                                                                                          *;
%* @macrotype LSAF                                                                          *;
%* @since 1.1                                                                               *;
%* @exposure external                                                                       *;
%*                                                                                          *;
%* @history 2013-08-30  updates for standardization and coding conventions                  *;
%* @history 2016-08-26  rebrand as LSAF                                                     *;
%*                                                                                          *;

%macro lsaf_objectexists(
  lsaf_path=
  ) /des='Does the object exist in the repository?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getproductname();

  %global _lsafMsg_
          _lsafObjectExists_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("exists", "%bquote(&lsaf_path)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine if the object exists: &lsaf_path." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (exists) then
      do;
        logMsg=" The object exists: &lsaf_path." ;
      end;
      else
      do;
        logMsg=" The object does not exist: &lsaf_path." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafObjectExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafObjectExists_", exists) ;

  run;

%mend lsaf_objectexists ;
