%* lsaf_processflowexists                                                                        *;
%*                                                                                               *;
%* Determines whether a process flow is defined for the specified context.                       *;
%*                                                                                               *;
%* The macro variable _lsafProcessFlowExists_ will be set to 1 if the process flow exists,       *;
%* 0 if it is not. In the event of an incorrect call to the macro which results in an error,     *;
%* the return value of _lsafProcessFlowExists_ will be set to -1.                                *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_PROCESSFLOWEXISTS (LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=lsaf-name);                    *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafProcessFlowExists_ indicates if the process flow exists in the context.          *;
%*                                                                                               *;
%* @param lsaf_path - required -  the path of the context object.                                *;
%* @param lsaf_process flow - required - the name of the process flow.                           *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-08-17  initial development                                                      *;
%*                                                                                               *;

%macro lsaf_processflowexists(
  lsaf_path=,
  lsaf_processflow=
  ) /des='Is the process flow defined in the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafProcessFlowExists_
          _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("exists", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the existence of process flow &lsaf_processflow in &lsaf_path.." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (exists) then
      do;
        logMsg=" The process flow &lsaf_processflow was found in &lsaf_path.";
      end;
      else
      do;
        logMsg=" The process flow &lsaf_processflow was not found in &lsaf_path.";
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafProcessFlowExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafProcessFlowExists_", exists) ;

  run;

%mend lsaf_processflowexists ;
