%* lsaf_removemember                                                                               *;
%*                                                                                                 *;
%* Removes a user or a group as a member of the specified organization, project, or analysis.      *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_REMOVEMEMBER(LSAF_PATH=lsaf-path, LSAF_MEMBER=lsaf-member-identifier                      *;
%* <, LSAF_TYPE=lsaf-type, LSAF_GROUP_CONTEXT=lsaf-group-context>);                                *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the context object (organization, project, or         *;
%*            analysis) to remove the member from.                                                 *;
%* @param lsaf_member - required - the id of the user or the name of the group being removed.      *;
%* @param lsaf_type - conditional - indicates the type of member. Valid values are USER and GROUP  *;
%*            (case-insensitive).  The default value is USER.                                      *;
%* @param lsaf_group_context - conditional - specifies the context in which the group being        *;
%*            removed is defined as a path (e.g., /SAS). This parameter is required if             *;
%*            LSAF_TYPE=GROUP. Otherwise, it will be ignored.                                      *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.2                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2013-08-30  updates for standardization and coding conventions                         *;
%* @history 2016-08-29  rebrand as LSAF                                                            *;
%*                                                                                                 *;

%macro lsaf_removemember(
  lsaf_path=,
  lsaf_member=,
  lsaf_type=USER,
  lsaf_group_context=
  ) /des='Remove a member from the context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/MembershipService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("removeMember", "%bquote(&lsaf_path)", "%bquote(&lsaf_type)", "%bquote(&lsaf_member)", "%bquote(&lsaf_group_context)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The &lsaf_type &lsaf_group_context &lsaf_member has been removed from &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot remove &lsaf_type  &lsaf_group_context &lsaf_member from &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_removemember ;
