%* lsaf_roleexists                                                                               *;
%*                                                                                               *;
%* Determines whether a a role is defined in the specified organization, project, or analysis.   *;
%*                                                                                               *;
%* The SAS macro variable _lsafRoleExists_ is set to a value of 1 when the role exists, 0 if it  *;
%* is not. In the event of an incorrect call to the macro which results in an error, the return  *;
%* value of _lsafRoleExists_ will be set to -1.                                                  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_ROLEEXISTS(LSAF_PATH=lsaf-path, LSAF_ROLE=lsaf-role-name);                              *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%* @macvar _lsafRoleExists_ indicates if the role exists in the context.                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the context object (organization, project, or       *;
%*            analysis).                                                                         *;
%* @param lsaf_role - required - the name of the role.                                           *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-26  rebrand as LSAF                                                          *;
%*                                                                                               *;

%macro lsaf_roleexists(
  lsaf_path=,
  lsaf_role=
  ) /des='Is the role defined in the context?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
          _lsafRoleExists_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;
    attrib exists          length=3;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RoleService") ;
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticBooleanMethod("exists", "%bquote(&lsaf_path)", "%bquote(&lsaf_role)", results, exists);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot determine the existence of role &lsaf_role in &lsaf_path.." ;
      exists=-1;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      if (exists) then
      do;
        logMsg=" The role &lsaf_role was found in &lsaf_path." ;
      end;
      else
      do;
        logMsg=" The role &lsaf_role was not found in &lsaf_path." ;
      end;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put logMsgPrefix " _lsafRoleExists_= " exists;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    call symputx("_lsafRoleExists_", exists);

  run;

%mend lsaf_roleexists ;
