%* lsaf_submitandpopulatewsjob                                                                       *;
%*                                                                                                   *;
%* Submits a SAS Life Science Analytics Framework workspace job for immediate execution and updates  *;
%* the input and output specifications in the job based on inputs and outputs accessed during the    *;
%* job run. The SAS macro variable _lsafPopulateWsJobSummissionId_ is set to the identifier for the  *;
%* job submission. If the job cannot be submitted, _lsafPopulateWsJobSummissionId_ will be set to    *;
%* blank.                                                                                            *;
%*                                                                                                   *;
%* If the job runs with SAS errors, the job will not be updated. If it runs successfully or with SAS *;
%* warnings, it will be updated, replacing any existing input and output definitions in the job.     *;
%*                                                                                                   *;
%* A data set representing the job parameters whose values are to be overridden for the current      *;
%* job submission can be sent into the call of lsaf_submitandpopulatewsjob. If no job parameters     *;
%* data set is sent in, the default values for all job parameters will be used.                      *;
%* The output data sets from either the lsaf_getjobparameters or lsaf_getworkspacejobparameters      *;
%* contains all the data relevant to the parameters for a specific job and can be used as a source   *;
%* for the input parameters data set for the lsaf_submitandpopulatewsjob macro. Modify the data set  *;
%* with any desired changes prior to calling lsaf_submitandpopulatewsjob.                            *;
%* - Rename the defaultValue variable to "value" or add a variable with the name "value" to          *;
%*   indicate the values to use for the current submission.                                          *;
%* - Default job parameter values will be used for missing parameters (deleted rows)                 *;
%* - Indicating an empty string for a parameter value will result in using a blank as the            *;
%*   parameter value for the current submission.                                                     *;
%* - Added parameters will be ignored.                                                               *;
%* - The parameters in the input data set are matched with the job parameters by name and type.      *;
%*   If no match is found for a parameter, its default value will be used.                           *;
%* - Data set rows with duplicate values for parameter name will cause an execution error.           *;
%* - DATE job parameter values must be in SAS datetime18. format (e.g., 02NOV2013:09:33:22), or      *;
%*   in SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).  For SAS datetime18. format, an     *;
%*   offset of zero hours from GMT will be used.  To specify an offset from GMT, use the             *;
%*   E8601DZ25. format.  Date values in formats other than the two described above will result       *;
%*   in an execution error.                                                                          *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_SUBMITANDPOPULATEWSJOB(LSAF_PATH=lsaf-path <, LSAF_USERELATIVEPATHS=use-relative-paths,     *;
%*                            SAS_DSNAME=SAS-data-set>);                                             *;
%*                                                                                                   *;
%* @macvar _lsafRC_  the return code.                                                                *;
%* @macvar _lsafMsg_ the return message.                                                             *;
%* @macvar _lsafPopulateWsJobSummissionId_ indicates the job submission identifier.                  *;
%*                                                                                                   *;
%* @param lsaf_path - required -the path of the job in the SAS Life Science Analytics Framework      *;
%*            workspace.                                                                             *;
%* @param lsaf_userelativepaths - optional - indicates whether or not to create generated inputs     *;
%*            and outputs as absolute or relative paths. The valid values are 0 and 1 to generate    *;
%*            absolute or relative paths respectively. The default is 0.                             *;
%* @param sas_dsname - optional - the name of the SAS data set containing the job parameter          *;
%*            information specified as libref.dataset. If not specified, the default job             *;
%*            parameters values will be used.  The data set must contain, at minimum, the            *;
%*            character variables listed below.  Any additional variables will be ignored.           *;
%*            - name: The name of the parameter.                                                     *;
%*            - type: The parameter type.  Valid values are CHARACTER, CHARACTER_MASKED,             *;
%*                  NUMERIC, DATE, FILE, and FOLDER.                                                 *;
%*            - value: The value of the parameter to be used in the current job submission.          *;
%*            - fileVersion: The file version to be used if the parameter is of type FILE.           *;
%*                  For versioned files, a blank value will result in the latest file version        *;
%*                  used.  For nonversioned files, the value is ignored.                             *;
%*                                                                                                   *;
%* @macrotype LSAF                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2016-10-20  initial coding                                                               *;
%*                                                                                                   *;

%macro lsaf_submitandpopulatewsjob(
   lsaf_path=,
   lsaf_userelativepaths=0,
   sas_dsname=
   ) /des='Submits a workspace job for execution and populates the inputs and outputs.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
          _lsafPopulateWsJobSummissionId_
   ;
   %local logMsgPrefix
          macroName
          notesOption
          parametersProvided
          _tempRUNWSJOB_tempDS_
   ;

   %let _lsafPopulateWsJobSummissionId_=;
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let parametersProvided=0;

   %*************************************************************;
   %* Turn off the notes generated for all the pre-processing.  *;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;
   %let notesOption=%sysfunc(getoption(notes));

   %if ( %CMPRES(&sas_dsname) NE %str() ) %then
   %do;

      %***************************************************************************;
      %* Validate the parameters data set if supplied.                           *;
      %* sets global macro variable _lsafRC_=0 if validation successful          *;
      %***************************************************************************;
      %lsaf_validate_parameters_dataset(callingMacro=&macroName,
                                      sas_dsn=&sas_dsname);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %GOTO PRINT_DSERROR_LOG_MSGS;
      %end;

      %let parametersProvided=1;
   %end;

   option notes;

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib
               returnCode        length=8
               message           length=$200
               jobSubmissionId   length=$525
      ;

      %if (&parametersProvided) %then
      %do;
         set &sas_dsname end=eof;

         if (_n_ = 1) then
         do;
      %end;

      %***********************************;
      %* Declare the java objects, once  *;
      %***********************************;
      declare javaobj params("com/sas/hls/macro/results/SasParameterValues");
      declare javaobj srv("com/sas/hls/macro/service/JobExecutionService");
      declare javaobj results("com/sas/hls/macro/results/Result");

      %if (&parametersProvided) %then
      %do;
         end;
         params.callVoidMethod("addValue", kstrip(name), kstrip(value), kstrip(type), kstrip(fileVersion));
         if (eof);
      %end;

      %***********************************;
      %* Submit the job                  *;
      %***********************************;
      srv.callStaticStringMethod("runAndPopulate", "%bquote(&lsaf_path)", params, "%bquote(&lsaf_userelativepaths)", results, jobSubmissionId);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
         logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Cannot submit job &lsaf_path.";
      end;
      else
      do;
         logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
         logMsg=" Submitted job &lsaf_path.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " " logMsg;
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      %if (&sas_dsname EQ %str()) %then
      %do;
         put logMsgPrefix "No job parameters were supplied.";
      %end;
      %else
      %do;
         put logMsgPrefix "sas_dsname = &sas_dsname";
      %end;
      put logMsgPrefix "_lsafPopulateWsJobSummissionId_= " jobSubmissionId;
      put;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);
      call symputx("_lsafPopulateWsJobSummissionId_", strip(jobSubmissionId));

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrefix  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put &logMsgPrefix Parameter data set = &sas_dsname;
      %put;

%mend lsaf_submitandpopulatewsjob;
