%* lsaf_syncfiletoworkspace                                                                       *;
%*                                                                                                *;
%* Gets the version of the SAS Life Science Analytics Framework from the repository file to       *;
%* the workspace.                                                                                 *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_SYNCFILETOWORKSPACE(LSAF_PATH=lsaf-path, LSAF_VERSION=lsaf-version>);                    *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required - the path of the file within the SAS Life Science Analytics       *;
%*            Framework repository to download.                                                   *;
%* @param lsaf_version - conditional - specifies the version of the file to be synchronized if    *;
%*            the file is versioned.  If the file is versioned and a version is specified the     *;
%*            value must be in the valid format including a decimal point such as 1.0 or 4.2.     *;
%*            If the file is versioned and no version is specified the latest version is          *;
%*            downloaded.  If the file is unversioned and a version is specified, the value will  *;
%*            be ignored.                                                                         *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.4                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2014-04-07 initial coding                                                             *;
%* @history 2016-08-26  rebrand as LSAF                                                           *;
%*                                                                                                *;

%macro lsaf_syncfiletoworkspace(
  lsaf_path=,
  lsaf_version=
  ) /des='Get latest or specific version of a SAS Life Science Analytics Framework repository file to the workspace.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode       length=8
           message          length=$200
           logMsgPrefix     length=$70
           logMsg           length=$300
    ;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("getFileToWS", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The repository object was synchronized to the workspace: &lsaf_path..";
    end;
    else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" The repository object could not be synchronized to the workspace: &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_syncfiletoworkspace;
