%* lsaf_updatecopytoworkspacestatus                                                               *;
%*                                                                                                *;
%* Updates the value for the Copy To Workspace status of a file or folder in the SAS Life Science *;
%* Analytics Framework repository. The following values are valid for the copy to workspace       *;
%* properties status.                                                                             *;
%*     ALLOW - Allow all                                                                          *;
%*     WARN  - Warn (excluding published jobs)                                                    *;
%*     DENY  - Deny (excluding published jobs)                                                    *;
%*     DENY_ALL  - Deny all                                                                       *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_UPDATECOPYTOWORKSPACESTATUS(LSAF_PATH=lsaf-path, LSAF_STATUS=lsaf-status);               *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param lsaf_path - required -  the path of the file.                                           *;
%* @param lsaf_status - required -  the copy to workspace status to set.                          *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2016-11-22  initial code                                                              *;
%*                                                                                                *;

%macro lsaf_updatecopytoworkspacestatus(
  lsaf_path=, lsaf_status=
  ) /des='Update the copy to workspace status of a SAS Life Science Analytics Framework file or folder.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("updateCopyToWorkspaceStatus", "%bquote(&lsaf_path)", "%bquote(%UPCASE(&lsaf_status))", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot update copy to workspace status: &lsaf_path." ;
    end;
    else
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" Copy to workspace status updated to &lsaf_status for &lsaf_path." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message;
    put logMsgPrefix " _lsafRC_= " returnCode;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_updatecopytoworkspacestatus;
