%* lsaf_updatefile                                                                                      *;
%*                                                                                                      *;
%* Updates the contents of an existing file in the SAS Life Science Analytics Framework repository.     *;
%*                                                                                                      *;
%* If the file is versioned a new version will be created.  If the file is not versioned, the           *;
%* existing contents will be overwritten.                                                               *;
%*                                                                                                      *;
%* NOTE: This macro is only supported from a PC SAS session. Using this macro within an active          *;
%* SAS Life Science Analytics Framework application session will result in a failure.                   *;
%*                                                                                                      *;
%* Syntax:                                                                                              *;
%* %LSAF_UPDATEFILE(LOCAL_PATH=local-path, LSAF_PATH=lsaf-path <, LSAF_COMMENT=lsaf-comment>,           *;
%* LSAF_VERSION=lsaf-version-type);                                                                     *;
%*                                                                                                      *;
%* @macvar _lsafRC_  the return code.                                                                   *;
%* @macvar _lsafMsg_ the return message.                                                                *;
%*                                                                                                      *;
%* @param local_path - required - the absolute path of the file on the local computer.                  *;
%* @param lsaf_path - required - the path of the content object in the SAS Life Science Analytics       *;
%*            Framework repository to update. The object must already exist and may not be a container. *;
%* @param lsaf_comment - conditional - the comment for a versioned file in the repository. This         *;
%*            value is ignored for non-versioned objects.                                               *;
%* @param lsaf_version - conditional - the type of version to be created if the file to be              *;
%*            updated is versioned. Valid values are MAJOR or MINOR. The default is MINOR. This         *;
%*            value is ignored for non-versioned objects.                                               *;
%*                                                                                                      *;
%* @macrotype LSAF                                                                                      *;
%* @since 1.1                                                                                           *;
%* @exposure external                                                                                   *;
%*                                                                                                      *;
%* @history 2013-08-30  updates for standardization and coding conventions                              *;
%* @history 2016-08-26  rebrand as LSAF                                                                 *;
%*                                                                                                      *;

%macro lsaf_updatefile(
  local_path=,
  lsaf_path=,
  lsaf_comment=,
  lsaf_version=
  ) /des='Update the contents of an existing SAS Life Science Analytics Framework object.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/RepositoryService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("updateFile", "%bquote(&local_path)", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", "%bquote(&lsaf_comment)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode eq 0) then
    do;
      logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
      logMsg=" The file was updated: &lsaf_path.." ;
    end;
    else
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot update file: &lsaf_path.." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put logMsgPrefix " local_path= &local_path" ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_updatefile ;
