%* lsaf_updatepfnotifrecipients                                                                   *;
%*                                                                                                *;
%* Updates the recipients for process flow notification elements using a data set as input.       *;
%*                                                                                                *;
%* The recipients for multiple notification elements can be modified using a single data set.     *;
%* Each recipient is represented as a single row in the input data set.                           *;
%*                                                                                                *;
%* The output data set from the lsaf_getpfnotificationrecipients macro can be used to determine   *;
%* the recipients currently set on the notification element.                                      *;
%* The output data set from the lsaf_getassignedmembers macro can be used to determine the        *;
%* principals (users and groups) available at the context level eligible to be recipients.        *;
%* The output data set from the lsaf_getprocessflowelements can be used to determine the ids for  *;
%* all notification elements for a process flow.                                                  *;
%*                                                                                                *;
%* The notification recipients represented in the input data set will replace the list of         *;
%* recipients currently saved on the notification element. A single row where TYPE and PRINCIPALID*;
%* are blank will result in clearing all notification recipients for the specified element.       *;
%*                                                                                                *;
%* Failure and Message Handling:                                                                  *;
%* The macro performs a series of validation procedures to determine that the input data set has  *;
%* the necessary structure for processing. If any of these validation procedures fail, an         *;
%* appropriate failure message is printed to the log and the macro stops additional processing.   *;
%*                                                                                                *;
%* In addition to other modifications, the macro sorts the input data set and stores the modified *;
%* data in a temporary data set called WORK._SASMACRO_NOTIRECIPIENTSTEMP__. In the event that a   *;
%* failure occurs during processing this temporary data set is retained as a reference.           *;
%* If all updates are performed successfully, this data set will be deleted from the work library.*;
%* A message will be printed to the log reporting whether the data set was retained or not.       *;
%* Regardless of success or failure of previous runs, this data set will be deleted at the        *;
%* beginning of each new run of the macro. The data values and sort order of the original input   *;
%* data set are maintained.                                                                       *;
%*                                                                                                *;
%* All records pertaining to the same notification element are processed as a single transaction  *;
%* by the macro. Messages will be printed to the log as each notification element is processed.   *;
%* If the update of a notification element fails, the macro will stop processing and a failure    *;
%* message will be written to the log. Successful updates to notification elements made prior to  *;
%* the observation containing the failure will be maintained in SAS Life Science Analytics        *;
%* Framework.                                                                                     *;
%*                                                                                                *;
%* If duplicate valid recipients are provided for the same element, only one of the values will   *;
%* be maintained.                                                                                 *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_UPDATEPFNOTIFRECIPIENTS(SAS_DSNAME=SAS-data-set);                                        *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param sas_dsname - required - the name of a SAS data set containing the recipients for the    *;
%*            notification elements to be modified specified as libref.dataset. The data set must *;
%*            contain, at minimum, the variables listed below.                                    *;
%*            Any additional variables will be ignored.                                           *;
%*            - processFlowPath: The path of the context object where the process flow is defined.*;
%*            - processFlowName: The name of the process flow.                                    *;
%*            - elementID: The id for the element applicable for the property.                    *;
%*            - type:  The recipient type.  Valid values are USER and GROUP.                      *;
%*            - principalId: The name of the recipient. The value is either a group name or a     *;
%*                  user ID.                                                                      *;
%*            - grpSrcCtxt: The context path in which a group recipient is defined. The value     *;
%*                  will be ignored if the recipient is of type user.                             *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2017-01-16  original coding                                                           *;
%*                                                                                                *;

%macro lsaf_updatepfnotifrecipients(
  sas_dsname=
  ) /des='Update the recipients for process flow notification elements.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local logMsgPrefix
         macroName
         tempDsMsg
         _tempUPDATE_tempDS_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

  %***************************************************************************;
  %* Cleanup from any previous runs that may have left temp data set behind. *;
  %***************************************************************************;
  %let _tempUPDATE_tempDS_=%str(_SASMACRO_NOTIRECIPIENTSTEMP__);

  %if (%sysfunc(exist(&_tempUPDATE_tempDS_)) > 0) %then
  %do;
    %put NOTE: &macroName: Deleting temporary data set &_tempUPDATE_tempDS_ before processing.;
    proc datasets library=work nolist;
      delete &_tempUPDATE_tempDS_;
    quit; run;
  %end;

  %***************************************************************************;
  %* Set local macro variables for processing.                               *;
  %*                                                                         *;
  %* setting a new macro variable for operation success because there are    *;
  %* operations in updateACL that could set _lsafRC_ to something other than *;
  %* 0 but still not be a failure.                                           *;
  %***************************************************************************;
  %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let tempDsMsg=%str(Temporary sorted data set work.&_tempUPDATE_tempDS_ will not be created.);

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(processFlowPath processFlowName elementID type principalId grpSrcCtxt)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &tempDsMsg ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  %************************;
  %* Sort the data        *;
  %************************;
  proc sort data=&sas_dsname
             out=&_tempUPDATE_tempDS_;
    by processFlowPath processFlowName elementID;
  run;

  option notes;

  %**********************************************;
  %* Process the records                        *;
  %**********************************************;
  data _null_;
    attrib returnCode    length=8
           message       length=$200
           updateCount   length=8
    ;

    set &_tempUPDATE_tempDS_ end=eof;
    by processFlowPath processFlowName elementID;

    %***********************************;
    %* Declare the java objects, once  *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj recipients("com/sas/hls/macro/results/SasProcessFlowElementPrincipals");
      declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService");
      declare javaobj results("com/sas/hls/macro/results/Result");

      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * " %sysfunc(lowcase("&SYSMACRONAME"));
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Input data set = &sas_dsname";
    end;

    if (first.elementID) then
    do;
      recipients.callVoidMethod("clearList");
      * do not want to add a blank recipient if the
      * user is trying to clear out the list.  ;
      if (last.elementID AND kstrip(type)="" AND kstrip(principalId)="") then
         GOTO DO_UPDATE;
    end;

    %***********************************;
    %* Create a new recipients list    *;
    %***********************************;
    recipients.callVoidMethod("addValue", kstrip(type), kstrip(principalId), kstrip(grpSrcCtxt));

    if (last.elementID) then
    do;
      GOTO DO_UPDATE;
    end;

    RETURN;

    DO_UPDATE:
      %***********************************;
      %* Call the update                 *;
      %***********************************;
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Processing element " elementID " in process flow " processFlowName " in "
          processFlowPath;
      srv.callStaticVoidMethod("updateElementPrincipals", kstrip(processflowpath), kstrip(processflowname), kstrip(elementID),
           "%bquote(SYSTEM_NOTIFICATION)", recipients, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * Unable to process " elementID;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafMsg_ = " message;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafRC_ = " returnCode;
        put;
        GOTO ENDPROCESS;
      end;
      else
      do;
         updateCount+1;
      end;

       if (eof) then
       do;
         GOTO ENDPROCESS;
       end;
    RETURN;


    ENDPROCESS:
    put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Successfully processed " updateCount "notification elements";
    put " ";

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

    STOP;

  run;

  %************************************************************;
  %* Cleanup temporary data set if the operation did not fail *;
  %************************************************************;
  %if (&_lsafRC_ eq 0) %then
  %do;
    %if (%sysfunc(exist(&_tempUPDATE_tempDS_))>0) %then
    %do;
      options notes;
      %put NOTE: &_LSAF_PRODUCT_NAME_ Macro: * %sysfunc(lowcase(&SYSMACRONAME)) is deleting temporary data set &_tempUPDATE_tempDS_;
      options nonotes;
      proc datasets library=work nolist;
        delete &_tempUPDATE_tempDS_;
        quit; run;
    %end;
  %end;
  %else
  %do;
    %put WARNING: &_LSAF_PRODUCT_NAME_ Macro: * Temporary sorted data set work.&_tempUPDATE_tempDS_ is not being deleted.;
  %end;

  options &notesOption;

%mend lsaf_updatepfnotifrecipients;
