%* lsaf_updatepftimerelements                                                                    *;
%*                                                                                               *;
%* Updates the values of process flow timer elements using a data set as input. Each row of the  *;
%* data set represents a timer element.                                                          *;
%*                                                                                               *;
%* The output data set from the lsaf_getpftimerelements macro can be used to determine the id    *;
%* and current values of the process flow timer elements. Rows where id matches an existing      *;
%* process flow timer element will be used to attempt to update the element. If the specified    *;
%* value is invalid for the element type, the update will be aborted and the appropriate error   *;
%* message will be printed to the log.                                                           *;
%*                                                                                               *;
%* Values of type DATE must be one of the following formats. Date values in formats other than   *;
%* the two described below will result in an execution error.                                    *;
%* - SAS datetime18. format (e.g., 02NOV2013:09:33:22) -  an offset of zero hours from GMT will  *;
%*   be used.                                                                                    *;
%* - SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).                                    *;
%*                                                                                               *;
%* Failure and Message Handling:                                                                 *;
%* The macro performs a series of validation procedures to determine that the input data set has *;
%* the necessary structure for processing. If any of these validation procedures fail, an        *;
%* appropriate failure message is printed to the log and the macro stops additional processing.  *;
%*                                                                                               *;
%* If a failure occurs while processing a specific timer element, the macro will stop processing *;
%* and a failure message will be written to the log. Successful updates made to elements prior   *;
%* to the element containing the failure will be maintained in SAS Life Science Analytics        *;
%* Framework.                                                                                    *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_UPDATEPFTIMERELEMENTS(SAS_DSNAME=SAS-data-set);                                         *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param sas_dsname - required - the name of a SAS data set containing the process flow data    *;
%*            values to be set on the process flow specified as libref.dataset.                  *;
%*            The data set must contain, at minimum, the variables listed below.  Any            *;
%*            additional variables will be ignored.                                              *;
%*            - processFlowPath: The path of the context object.                                 *;
%*            - processFlowName: The name of the process flow.                                   *;
%*            - elementID: The id for the element.                                               *;
%*            - value: The value of a date timer. This is ignored when timer type is DURATION.   *;
%*            - years: Years value for duration timer. This is ignored when timer type is DATE.  *;
%*            - months: Months value for duration timer. This is ignored when timer type is DATE.*;
%*            - days: Days value duration timer. This is ignored when timer type is DATE.        *;
%*            - hours: Hours value for duration timer. This is ignored when timer type is DATE.  *;
%*            - minutes: Minutes value for duration timer. This is ignored when timer type is    *;
%*                 DATE.                                                                         *;
%*            - seconds: Seconds value for duration timer. This is ignored when timer type is    *;
%*                 DATE.                                                                         *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2017-01-03  original code                                                            *;
%*                                                                                               *;

%macro lsaf_updatepftimerelements(
  sas_dsname=
  ) /des='Updates timer elements for process flows in the SAS Life Science Analytics Framework.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %***************************************************************************;
  %* Set local macro variables for processing.                               *;
  %***************************************************************************;
  %local logMsgPrefix
         macroName
         ;

  %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(processFlowPath processFlowName elementID value years months days hours minutes seconds),
        required_numeric_variables=%str(years months days hours minutes seconds),
        allow_empty_datasets=0);

  %if (&_lsafRC_ ne 0) %then
  %do;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  option notes;

  %**********************************************;
  %* Process the records                        *;
  %**********************************************;
  data _null_;
    attrib returnCode       length=8
           message          length=$200
           updatedCnt       length=8
    ;

    set &sas_dsname end=eof;

    %***********************************;
    %* Declare the java objects, once  *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService");
      declare javaobj results("com/sas/hls/macro/results/Result");
      declare javaobj timer("com/sas/hls/macro/results/SasProcessFlowTimerElement");

      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * " %sysfunc(lowcase("&SYSMACRONAME"));
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Input data set = &sas_dsname";
    end;

    %***********************************;
    %* Set timer values                *;
    %***********************************;
    timer.callVoidMethod("setTimerValues", kstrip(value),
                                           years, months, days, hours, minutes, seconds);

    %***********************************;
    %* Call the update                 *;
    %***********************************;
    put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Processing " elementID "for " processFlowName
          " in " processFlowPath;
    srv.callStaticVoidMethod("updateTimerElement", kstrip(processFlowPath), kstrip(processFlowName),
         kstrip(elementID), timer, results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    if (returnCode ne 0) then
    do;
      put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * Unable to process " elementID "for " processFlowName
          " in " processFlowPath ".";
      put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafMsg_ = " message;
      put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafRC_ = " returnCode;
      put;
      GOTO ENDPROCESS;
    end;
    else
    do;
      updatedCnt+1;
    end;

    if (eof) then
    do;
      GOTO ENDPROCESS;
    end;

    return;

    ENDPROCESS:
    put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Successfully processed " updatedCnt "timer elements";
    put " ";

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);
    STOP;

  run;

  options &notesOption;

%mend lsaf_updatepftimerelements;
