%* lsaf_updatepfuserelementinfo                                                                   *;
%*                                                                                                *;
%* Updates the properties for process flow user elements using a data set as input.               *;
%*                                                                                                *;
%* Properties for multiple user elements can be modified using a single data set. Each property   *;
%* is represented as a single row in the input data set.                                          *;
%*                                                                                                *;
%* The output data set from the lsaf_getpfuserelementinfo macro can be used to determine the name *;
%* and current values of the user element properties. Rows where name matches an existing user    *;
%* element property will be used to attempt to update the property. If the specified value is     *;
%* invalid for the property type, the update will be aborted and the appropriate error message    *;
%* will be printed to the log.                                                                    *;
%*                                                                                                *;
%* Failure and Message Handling:                                                                  *;
%* The macro performs a series of validation procedures to determine that the input data set has  *;
%* the necessary structure for processing. If any of these validation procedures fail, an         *;
%* appropriate failure message is printed to the log and the macro stops additional processing.   *;
%*                                                                                                *;
%* In addition to other modifications, the macro sorts the input data set and stores the modified *;
%* data in a temporary data set called WORK.__SASMACRO_USERELEMENTSTEMP__. In the event that a    *;
%* failure occurs during processing this temporary data set is retained as a reference. If all    *;
%* updates are performed successfully, this data set will be deleted from the work library. A     *;
%* message will be printed to the log reporting whether the data set was retained or not.         *;
%* Regardless of success or failure of previous runs, this data set will be deleted at the        *;
%* beginning of each new run of the macro. The data values and sort order of the original input   *;
%* data set are maintained.                                                                       *;
%*                                                                                                *;
%* All records pertaining to the same user element are processed as a single transaction by the   *;
%* macro. Messages will be printed to the log as each user element is processed. If the update of *;
%* a user element fails, the macro will stop processing and a failure message will be written to  *;
%* the log. Successful updates to user elements made prior to the observation containing the      *;
%* failure will be maintained in SAS Life Science Analytics Framework.                            *;
%*                                                                                                *;
%* Syntax:                                                                                        *;
%* %LSAF_UPDATEPFUSERELEMENTINFO(SAS_DSNAME=SAS-data-set);                                        *;
%*                                                                                                *;
%* @macvar _lsafRC_  the return code.                                                             *;
%* @macvar _lsafMsg_ the return message.                                                          *;
%*                                                                                                *;
%* @param sas_dsname - required - the name of a SAS data set containing the properties            *;
%*            information for all the user element properties to be modified specified as         *;
%*            libref.dataset. The data set must contain, at minimum, the variables listed below.  *;
%*            Any additional variables will be ignored.                                           *;
%*            - processFlowPath: The path of the context object where the process flow is defined.*;
%*            - processFlowName: The name of the process flow.                                    *;
%*            - elementID: The id for the element applicable for the property.                    *;
%*            - name: The name of the property.                                                   *;
%*            - value: The value of the property.                                                 *;
%*                                                                                                *;
%* @macrotype LSAF                                                                                *;
%* @since 1.5                                                                                     *;
%* @exposure external                                                                             *;
%*                                                                                                *;
%* @history 2017-01-18  original coding                                                           *;
%*                                                                                                *;

%macro lsaf_updatepfuserelementinfo(
  sas_dsname=
  ) /des='Update properties for process flow user elements.';

  %*************************************************************;
  %* Turn off the notes generated for all the pre-processing.  *;
  %* Save original option setting so it can be reset properly. *;
  %*************************************************************;
  %let notesOption=%sysfunc(getoption(notes));
  options nonotes;

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %local logMsgPrefix
         macroName
         tempDsMsg
         _tempUPDATE_tempDS_
  ;

  %let _lsafRC_ =%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);
  %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

  %***************************************************************************;
  %* Cleanup from any previous runs that may have left temp data set behind. *;
  %***************************************************************************;
  %let _tempUPDATE_tempDS_=%str(__SASMACRO_USERELEMENTSTEMP__);

  %if (%sysfunc(exist(&_tempUPDATE_tempDS_)) > 0) %then
  %do;
    %put NOTE: &macroName: Deleting temporary data set &_tempUPDATE_tempDS_ before processing.;
    proc datasets library=work nolist;
      delete &_tempUPDATE_tempDS_;
    quit; run;
  %end;

  %***************************************************************************;
  %* Set local macro variables for processing.                               *;
  %*                                                                         *;
  %* setting a new macro variable for operation success because there are    *;
  %* operations in updateACL that could set _lsafRC_ to something other than *;
  %* 0 but still not be a failure.                                           *;
  %***************************************************************************;
  %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
  %let tempDsMsg=%str(Temporary sorted data set work.&_tempUPDATE_tempDS_ will not be created.);

  %***************************************************************************;
  %* Validate the input data set                                             *;
  %* sets global macro variable _lsafRC_=0 if validation successful          *;
  %***************************************************************************;
  %lsaf_validate_sasdataset(callingMacro=&macroName,
        sas_dsn=&sas_dsname,
        required_variables=%str(processFlowPath processFlowName elementID name value)
        );
  %if (&_lsafRC_ ne 0) %then
  %do;
    %put &logMsgPrefix &macroName ;
    %put &logMsgPrefix Input data set = &sas_dsname;
    %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
    %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
    %put &logMsgPrefix &tempDsMsg ;
    %put;
    %return;
  %end;

  %***************************************************************************;
  %* If data set is valid, process it.                                       *;
  %***************************************************************************;
  %put NOTE: &macroName: Processing data set &sas_dsname;

  %************************;
  %* Sort the data        *;
  %************************;
  proc sort data=&sas_dsname
             out=&_tempUPDATE_tempDS_;
    by processFlowPath processFlowName elementID;
  run;

  option notes;

  %**********************************************;
  %* Process the records                        *;
  %**********************************************;
  data _null_;
    attrib returnCode    length=8
           message       length=$200
           updateCount   length=8
    ;

    set &_tempUPDATE_tempDS_ end=eof;
    by processFlowPath processFlowName elementID;

    %***********************************;
    %* Declare the java objects, once  *;
    %***********************************;
    if (_n_ = 1) then
    do;
      declare javaobj props("com/sas/hls/macro/results/SasProperties");
      declare javaobj srv("com/sas/hls/macro/service/ProcessFlowSetupService");
      declare javaobj results("com/sas/hls/macro/results/Result");

      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * " %sysfunc(lowcase("&SYSMACRONAME"));
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Input data set = &sas_dsname";
    end;

    if (first.elementID) then
    do;
      props.callVoidMethod("clearList");
    end;

    %***********************************************;
    %* Add to list of properties for the element.  *;
    %***********************************************;
    props.callVoidMethod("addProperty", kstrip(name), kstrip(value));

    if (last.elementID) then
    do;

      %***********************************;
      %* Call the update                 *;
      %***********************************;
      put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Processing element " elementID " in process flow " processFlowName " in "
          processFlowPath;

      srv.callStaticVoidMethod("updateElementProperties", kstrip(processflowpath), kstrip(processflowname), kstrip(elementID),
          "%bquote(USER_TASK)", props, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * Unable to process " elementID;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafMsg_ = " message;
        put "ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: * _lsafRC_ = " returnCode;
        put;
        GOTO ENDPROCESS;
      end;
      else
      do;
         updateCount+1;
      end;
    end;

    if (eof) then
    do;
      GOTO ENDPROCESS;
    end;

    return;

    ENDPROCESS:
    put "NOTE: &_LSAF_PRODUCT_NAME_ Macro: * Successfully processed " updateCount "user elements";
    put " ";

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

    STOP;

  run;

  %************************************************************;
  %* Cleanup temporary data set if the operation did not fail *;
  %************************************************************;
  %if (&_lsafRC_ eq 0) %then
  %do;
    %if (%sysfunc(exist(&_tempUPDATE_tempDS_)) EQ 1) %then
    %do;
      %put NOTE: &_LSAF_PRODUCT_NAME_ Macro: * %sysfunc(lowcase(&SYSMACRONAME)) is deleting temporary data set &_tempUPDATE_tempDS_;
      option nonotes;
      proc datasets library=work nolist;
        delete &_tempUPDATE_tempDS_;
        quit; run;
    %end;
  %end;
  %else
  %do;
    %put WARNING: &_LSAF_PRODUCT_NAME_ Macro: * Temporary sorted data set work.&_tempUPDATE_tempDS_ is not being deleted.;
  %end;

  options &notesOption;

%mend lsaf_updatepfuserelementinfo;
