%* lsaf_updateprocessflowproperties                                                              *;
%*                                                                                               *;
%* Updates a process flow in the SAS Life Science Analytics Framework repository.                *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %LSAF_UPDATEPROCESSFLOWPROPERTIES(LSAF_PATH=lsaf-path, LSAF_PROCESSFLOW=process-flow-name,    *;
%*           SAS_DSNAME=SAS-data-set);                                                           *;
%*                                                                                               *;
%* @macvar _lsafRC_  the return code.                                                            *;
%* @macvar _lsafMsg_ the return message.                                                         *;
%*                                                                                               *;
%* @param lsaf_path - required - the path of the process flow being updated.                     *;
%* @param lsaf_processflow - required - the name of the process flow being updated.              *;
%* @param sas_dsname - required - the name of a SAS data set containing the property             *;
%*            values to be set on the process flow specified as libref.dataset.                  *;
%*            The data set must contain, at minimum, the variables listed below.  Any            *;
%*            additional variables will be ignored.                                              *;
%*            - name: the name of the property to be updated.                                    *;
%*            - value: the value to set on the property.                                         *;
%*            The output data set from the lsaf_getprocessflowproperties macro can be used to    *;
%*            determine the name and current values of the process flow properties.              *;
%*            Rows where name matches an existing process flow property will be used to          *;
%*            attempt to update the property. If the specified value is invalid for the property *;
%*            type, the update will be aborted and the appropriate error message will be printed *;
%*            to the log.                                                                        *;
%*            Values of type DATE must be one of the following formats. Date values in formats   *;
%*            other than the two described below will result in an execution error.              *;
%*            - SAS datetime18. format (e.g., 02NOV2013:09:33:22) -  an offset of zero hours     *;
%*              from GMT will be used.                                                           *;
%*            - SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).                         *;
%*                                                                                               *;
%* @macrotype LSAF                                                                               *;
%* @since 1.5                                                                                    *;
%* @exposure external                                                                            *;
%*                                                                                               *;
%* @history 2016-12-06  initial programming                                                      *;
%*                                                                                               *;

%macro lsaf_updateprocessflowproperties(
  lsaf_path=,
  lsaf_processflow=,
  sas_dsname=_lsaf_updatepf_donotupdate_
  ) /des='Updates a process flow in the specified SAS Life Science Analytics Framework context.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           notesOption
   ;

   %local logMsgPrefix
          macroName
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly. Unknown err%str(or).);
   %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );
   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));

   %if (&sas_dsname=_lsaf_updatepf_donotupdate_) %then
   %do;
         %let _lsafMsg_= %str(No data has been provided for the update.);
         %GOTO PRINT_DSERROR_LOG_MSGS;
   %end;

   %*************************************************************;
   %* Save original option setting so it can be reset properly. *;
   %*************************************************************;
   %let notesOption=%sysfunc(getoption(notes));

   %if ( %CMPRES(&sas_dsname) NE %str(_lsaf_updatepf_donotupdate_) ) %then
   %do;
      %***************************************************************************;
      %* Validate the Process Flow Data data set if supplied.                    *;
      %* sets global macro variable _lsafRC_=0 if validation successful          *;
      %***************************************************************************;
      %lsaf_validate_sasdataset(callingMacro=&macroName,
                                      sas_dsn=&sas_dsname,
                                      required_variables=%str(name value),
                                      allow_empty_datasets=0);
      %if (&_lsafRC_ ne 0) %then
      %do;
         %let _lsafMsg_= sas_dsname: &_lsafMsg_;
         %GOTO PRINT_DSERROR_LOG_MSGS;
      %end;
   %end;

   option notes;

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib returnCode      length=8
              message         length=$200
              logMsgPrefix    length=$70
              logMsg          length=$300
              logMsg          length=$300
              updatedName     length=$300
      ;

      set &sas_dsname end=eof;
      if (_n_ EQ 1) then
      do;
          %***********************************;
          %* Declare the java objects        *;
          %***********************************;
          declare javaobj srv("com/sas/hls/macro/service/ProcessFlowService");
          declare javaobj results("com/sas/hls/macro/results/Result");
          declare javaobj data("com/sas/hls/macro/results/SasProperties");
      end;

      data.callVoidMethod("addProperty", kstrip(name), kstrip(value));

      if (eof);

      %***********************************;
      %* Attempt the operation           *;
      %***********************************;
      srv.callStaticStringMethod("updateProperties", "%bquote(&lsaf_path)", "%bquote(&lsaf_processflow)",
                                           data, results, updatedName);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
         logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Cannot update process flow properties for &lsaf_processflow in &lsaf_path.." ;
      end;
      else
      do;
         logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
         logMsg=" The process flow properties were updated for &lsaf_processflow was updated in &lsaf_path.." ;
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " " logMsg;
      updatedName=kstrip(updatedName);
      if (returnCode=0 AND kstrip("&lsaf_processflow") NE updatedName) then
      do;
         put logMsgPrefix " The process flow has been renamed to " updatedName ".";
      end;
      put logMsgPrefix " _lsafMsg_= " message ;
      put logMsgPrefix " _lsafRC_= " returnCode ;
      put;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      options &notesOption;
      %put &logMsgPrefix  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put;
      options &notesOption;

%mend lsaf_updateprocessflowproperties;
