%* lsaf_updateprojectstate                                                                         *;
%*                                                                                                 *;
%* Update the state of a project in the SAS Life Science Analytics Framework repository.           *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %LSAF_UPDATEPROJECTSTATE(LSAF_PATH=lsaf-path, LSAF_STATE=lsaf-state,                            *;
%* < LSAF_COMMENT=lsaf-comment, LSAF_INCLUDE_ANALYSES=lsaf-include-analyses >);                    *;
%*                                                                                                 *;
%* @macvar _lsafRC_  the return code.                                                              *;
%* @macvar _lsafMsg_ the return message.                                                           *;
%*                                                                                                 *;
%* @param lsaf_path - required - the path of the project to be updated.                            *;
%* @param lsaf_state - required - the state to set on the project.                                 *;
%*            Values:  DEV | PROD | CLOSED                                                         *;
%* @param lsaf_comment - optional - the reason for the state change.                               *;
%* @param lsaf_include_analyses - optional - whether project child analyses, if any exist, should  *;
%*            have their state updated as well.                                                    *;
%*            Values: 0 | 1                                                                        *;
%*            If no value is supplied, the child analysis states will be updated to the same       *;
%*            state value as supplied for the project.                                             *;
%*                                                                                                 *;
%* @macrotype LSAF                                                                                 *;
%* @since 1.4                                                                                      *;
%* @exposure external                                                                              *;
%*                                                                                                 *;
%* @history 2014-05-01 initial coding                                                              *;
%* @history 2016-08-26 rebrand as LSAF                                                             *;
%* @history 2016-11-08 updated messaging                                                           *;
%*                                                                                                 *;

%macro lsaf_updateprojectstate(
  lsaf_path=,
  lsaf_state=,
  lsaf_comment=,
  lsaf_include_analyses=1
  ) /des='Update the state of a project.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %lsaf_getProductName();

  %global _lsafMsg_
          _lsafRC_
  ;

  %let _lsafRC_=%str(-999);
  %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %**********************************************;
  %* Start the datastep to call the java code   *;
  %**********************************************;
  data _null_;
    attrib returnCode      length=8;
    attrib message         length=$200;
    attrib logMsgPrefix    length=$70;
    attrib logMsg          length=$300;

    %***********************************;
    %* Declare the java objects        *;
    %***********************************;
    declare javaobj srv("com/sas/hls/macro/service/ProjectService");
    declare javaobj results("com/sas/hls/macro/results/Result");

    %***********************************;
    %* Attempt the operation           *;
    %***********************************;
    srv.callStaticVoidMethod("updateState", "%bquote(&lsaf_path)", "%bquote(&lsaf_state)", "%bquote(&lsaf_comment)",
              "%bquote(&lsaf_include_analyses)", results);

    %***********************************;
    %* Retrieve the results            *;
    %***********************************;
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);

    %***********************************;
    %* Process the results             *;
    %***********************************;
    logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
    logMsg=" The state for project &lsaf_path was updated to &lsaf_state.." ;
    if (returnCode ne 0) then
    do;
      logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      logMsg=" Cannot update project &lsaf_path." ;
    end;

    %***********************************;
    %* Print messages in log           *;
    %***********************************;
    put;
    put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
    put logMsgPrefix " " logMsg;
    put logMsgPrefix " _lsafMsg_= " message ;
    put logMsgPrefix " _lsafRC_= " returnCode ;
    put;

    %***********************************;
    %* Set the macro variables         *;
    %***********************************;
    call symputx("_lsafRC_", returnCode);
    call symputx("_lsafMsg_", message);

  run;

%mend lsaf_updateprojectstate;
