%* lsaf_updateworkspacejob                                                                           *;
%*                                                                                                   *;
%* Updates an existing job in the SAS Life Science Analytics Framework workspace.                    *;
%*                                                                                                   *;
%* Data sets representing the job data to be updated must be sent into the call to                   *;
%* lsaf_updateworkspacejob. The information in the data sets will replace the data of the existing   *;
%* job. The output data sets from the following macros contain all the data relevant to the job      *;
%* properties that can be updated. See the macro documentation for more details.                     *;
%* - lsaf_getjobinfo or lsaf_getworkspacejobinfo                                                     *;
%* - lsaf_getjobtasks or lsaf_getworkspacejobtasks                                                   *;
%* - lsaf_getjobparameters or lsaf_getworkspacejobparameters                                         *;
%* - lsaf_getjobinputs or lsaf_getworkspacejobinputs                                                 *;
%* - lsaf_getjoboutputs or lsaf_getworkspacejoboutputs                                               *;
%*                                                                                                   *;
%* Syntax:                                                                                           *;
%* %LSAF_UPDATEWORKSPACEJOB(LSAF_PATH=lsaf-path <SAS_DSNAME_INFO=SAS-data-set-info,                  *;
%*      SAS_DSNAME_TASKS=SAS-data-set-jobtasks, SAS_DSNAME_PARAMETERS=SAS-data-set-parameters,       *;
%*      SAS_DSNAME_INPUTS=SAS-data-set-inputs, SAS_DSNAME_OUTPUTS=SAS-data-set-outputs>);            *;
%*                                                                                                   *;
%* @macvar _lsafRC_  the return code.                                                                *;
%* @macvar _lsafMsg_ the return message.                                                             *;
%*                                                                                                   *;
%* @param lsaf_path - required -the path for the job to be updated in the SAS Life Science Analytics *;
%*             Framework workspace.                                                                  *;
%* @param sas_dsname_info - optional - the name of the SAS data set containing the job               *;
%*            information specified as libref.dataset. The data set must contain, at minimum, the    *;
%*            character variables listed below.  Any additional variables will be ignored.           *;
%*            - name: The name of the job property.                                                  *;
%*            - value: The value of the property to be specified.                                    *;
%*            The information that can be specified is DESCRIPTION, LOGPATH, LSTPATH, MANIFESTPATH,  *;
%*            NEWOUTPUTS_CUSTOMVERSION, NEWOUTPUTS_ENABLEVERSION, and NEWOUTPUTS_VERSIONTYPE.        *;
%*            All other values listed in the data set will be ignored. All data are case-insensitive.*;
%*            If a name is listed more than once, the last value processed will be used. If a name   *;
%*            is omitted, the value in the job will not be updated.                                  *;
%* @param sas_dsname_tasks - optional - the name of the SAS data set containing the list of          *;
%*            paths to the SAS programs to be run specified as libref.dataset. If the data set is    *;
%*            empty, all tasks will be removed from the job. The data set must contain, at minimum, *;
%*            the character variables listed below.  Any additional variables will be ignored.       *;
%*            - taskPath: The path of the SAS program file.                                          *;
%*            - taskVersion: The version of the SAS program file. Blank and single asterisk values   *;
%*                  will indicate the latest file version.                                           *;
%* @param sas_dsname_parameters - optional - the name of the SAS data set containing the job         *;
%*            parameter information specified as libref.dataset. If the data set is empty, all job   *;
%*            parameters will be removed from the job. The data set must contain, at minimum, the    *;
%*            character variables listed below.  Any additional variables will be ignored.           *;
%*            - name: The name of the parameter.                                                     *;
%*            - label: The label for the parameter.                                                  *;
%*            - type: The parameter type.  Valid values are CHARACTER, CHARACTER_MASKED,             *;
%*                  NUMERIC, DATE, FILE, and FOLDER.                                                 *;
%*            - defaultValue: The value to be set as the default value for the parameter.            *;
%*                  DATE job parameter values must be one of the following formats. Date values in   *;
%*                  formats other than the two described below will result in an execution error.    *;
%*                  - SAS datetime18. format (e.g., 02NOV2013:09:33:22) -  an offset of zero hours   *;
%*                         from GMT will be used.                                                    *;
%*                  - SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).                       *;
%*            - fileVersion: The file version to be used if the parameter is of type FILE. Blank and *;
%*                         single asterisk values will indicate the latest file version.             *;
%*            Data set rows with duplicate values for parameter name will result in a data set       *;
%*            validation error.                                                                      *;
%* @param sas_dsname_inputs - optional - the name of the SAS data set containing the list of job     *;
%*            inputs specified as libref.dataset. If the data set is empty, all job inputs will be   *;
%*            removed from the job. The data set must contain, at minimum, the variables listed      *;
%*            below as the type specified. Any additional variables will be ignored.                 *;
%*            - inputPath (character): The path of the file or container to be used as input.        *;
%*            - inputType (character): The type of input object. Valid values are FILE and CONTAINER.*;
%*            - inputVersion (character): The version of the FILE object. Blank and single asterisk  *;
%*                  values will indicate the latest file version. The value is ignored for inputs of *;
%*                  type CONTAINER.                                                                  *;
%*            - inputIncludesSubFolders (numeric): A flag indicating whether or not the input folder *;
%*                  should include subfolders. The valid values are 0 and 1. The value is ignored    *;
%*                  for inputs of type FILE.                                                         *;
%*            Data set rows with duplicate values for inputPath will result in a data set            *;
%*            validation error.                                                                      *;
%* @param sas_dsname_outputs - optional - the name of the SAS data set containing the list of job    *;
%*            outputs specified as libref.dataset. If the data set is empty, all job outputs will be *;
%*            removed from the job. The data set must contain, at minimum, the variables listed      *;
%*            below as the type specified. Any additional variables will be ignored.                 *;
%*            - outputPath (character): The path of the container to be used as output.              *;
%*            - outputIncludesSubFolders (numeric): A flag indicating whether or not the output      *;
%*                  container should include subfolders. The valid values are 0 and 1.               *;
%*            Data set rows with duplicate values for outputPath will result in a data set           *;
%*            validation error.                                                                      *;
%*                                                                                                   *;
%* @since 1.5                                                                                        *;
%* @exposure external                                                                                *;
%*                                                                                                   *;
%* @history 2016-09-23  initial coding                                                               *;
%*                                                                                                   *;

%macro lsaf_updateworkspacejob(
   lsaf_path=,
   sas_dsname_info=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_tasks=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_parameters=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_inputs=%str(_lsaf_updateworkspacejob_ds_notprovided_),
   sas_dsname_outputs=%str(_lsaf_updateworkspacejob_ds_notprovided_)
   ) /des='Update a job in the workspace.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
  %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
           notesOption
   ;

   %local haveData
          haveObs
          logMsgPrefix
          macroName
   ;

   %let haveData =%str(1);
   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly. Unknown err%str(or).);
   %let _lsafWorkspaceJobPath_=;

   %let notesOption=%sysfunc(getoption(notes));

   %*********************************************************************************;
   %* Validate all input data sets and update master data set to set server values. *;
   %* Initializes _lsafMASTERJobDATA_tempDS_input macro variable.                   *;
   %*********************************************************************************;
   %let macroName=%sysfunc(lowcase(&SYSMACRONAME));
   %lsaf_process_jobdatasets(callingMacro=&macroName,
                              sas_dsname_info=&sas_dsname_info,
                              sas_dsname_tasks=&sas_dsname_tasks,
                              sas_dsname_parameters=&sas_dsname_parameters,
                              sas_dsname_inputs=&sas_dsname_inputs,
                              sas_dsname_outputs=&sas_dsname_outputs
   );
   %let logMsgPrefix=%str(ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro: );

   options notes;
   %if (&_lsafRC_ ne 0) %then
   %do;
      %GOTO PRINT_DSERROR_LOG_MSGS;
   %end;

   %if (&_lsafMASTERJobDATA_tempDS_ EQ %str(_null_)) %then
   %do;
         %LET haveData=0;
   %end;

   %LET haveObs=0;

   %let _lsafMASTERJobDATA_ID_ = %sysfunc( open(&_lsafMASTERJobDATA_tempDS_) );
   %let _lsafMASTERJobDATA_NOBS_ = %sysfunc( attrn(&_lsafMASTERJobDATA_ID_,nobs) );
   %let _lsafMASTERJobDATA_RC_ = %sysfunc( close(&_lsafMASTERJobDATA_ID_) );

   %if (&_lsafMASTERJobDATA_NOBS_ GE 1) %then
   %do;
        %LET haveObs=1;
   %end;

   %let _lsafRC_ =%str(-999);

   %**********************************************;
   %* Process the records                        *;
   %**********************************************;
   data _null_;
      attrib
               returnCode        length=8
               message           length=$200
      ;

      %if (&haveObs EQ 1) %then
      %do;
      set &_lsafMASTERJobDATA_tempDS_ end=eof;

      if (_n_ EQ 1) then
      do;
      %end;
         %***********************************;
         %* Declare the java objects, once  *;
         %***********************************;
         declare javaobj srv("com/sas/hls/macro/service/JobService");
         declare javaobj results("com/sas/hls/macro/results/Result");
         declare javaobj info("com/sas/hls/macro/results/SasJobInfo");
         declare javaobj tasks("com/sas/hls/macro/results/SasJobResourceSpecifications");
         declare javaobj params("com/sas/hls/macro/results/SasJobParameters");
         declare javaobj inputs("com/sas/hls/macro/results/SasJobResourceSpecifications");
         declare javaobj outputs("com/sas/hls/macro/results/SasJobResourceSpecifications");

         %if (&haveData EQ 1) %then
         %do;
            setData="1";
            %if (&sas_dsname_tasks NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
            %do;
               tasks.callVoidMethod("setUpdateData", kstrip(setData));
            %end;
            %if (&sas_dsname_parameters NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
            %do;
               params.callVoidMethod("setUpdateData", kstrip(setData));
            %end;
            %if (&sas_dsname_inputs NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
            %do;
               inputs.callVoidMethod("setUpdateData", kstrip(setData));
            %end;
            %if (&sas_dsname_outputs NE %str(_lsaf_updateworkspacejob_ds_notprovided_)) %then
            %do;
               outputs.callVoidMethod("setUpdateData", kstrip(setData));
            %end;
         %end;

      %if (&haveObs EQ 1) %then
      %do;
      end;  * ends if (_n_=1);
      if (source EQ "&source_info") then
      do;
         name=kstrip(lowcase(name));
         select (name);
            when ("description") info.callVoidMethod("setDescription", kstrip(value));
            when ("logpath") info.callVoidMethod("setLogPath", kstrip(value));
            when ("lstpath") info.callVoidMethod("setLstPath", kstrip(value));
            when ("manifestpath") info.callVoidMethod("setManifestPath", kstrip(value));
            when ("newoutputs_customversion") info.callVoidMethod("setCustomVersion", kstrip(value));
            when ("newoutputs_enableversion") info.callVoidMethod("setEnableVersioning", kstrip(value));
            when ("newoutputs_versiontype") info.callVoidMethod("setVersionType", kstrip(upcase(value)));
            otherwise;
         end;
      end;
      else if (source EQ "&source_parms") then
      do;
         params.callVoidMethod("addValue", kstrip(name), kstrip(pLabel), kstrip(type), kstrip(value), kstrip(version));
      end;
      else if (source EQ "&source_tasks") then
      do;
         tasks.callVoidMethod("addTaskValue", kstrip(value), kstrip(version));
      end;
      else if (source EQ "&source_inputs") then
      do;
         inputs.callVoidMethod("addInputValue", kstrip(value), kstrip(version), includeSubs, kstrip(type));
      end;
      else if (source EQ "&source_outputs") then
      do;
         outputs.callVoidMethod("addOutputValue", kstrip(value), includeSubs);
      end;

      %***********************************;
      %* Attempt the operation           *;
      %***********************************;
      if (eof);
      %end;

      srv.callStaticVoidMethod("updateWorkspaceJob", "%bquote(&lsaf_path)", info, tasks, params, inputs, outputs, results);

      %***********************************;
      %* Retrieve the results            *;
      %***********************************;
      results.callIntMethod("getReturnCode", returnCode);
      results.callStringMethod("getReturnMessage", message);

      %***********************************;
      %* Process the results             *;
      %***********************************;
      if (returnCode ne 0) then
      do;
         logMsgPrefix="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
         logMsg=" Could not update job in the workspace: &lsaf_path.";
      end;
      else
      do;
         logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
         logMsg=" Updated job in workspace using path &lsaf_path.";
      end;

      %***********************************;
      %* Print messages in log           *;
      %***********************************;
      put;
      put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
      put logMsgPrefix " " logMsg;
      put logMsgPrefix " _lsafMsg_= " message;
      put logMsgPrefix " _lsafRC_= " returnCode;

      %if (&haveData NE 1) %then
      %do;
         put logMsgPrefix " No job data was provided.";
      %end;
      %else
      %do;
         %if (&sas_dsname_info NE _lsaf_updateworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_info = &sas_dsname_info";
         %end;
         %if (&sas_dsname_tasks NE _lsaf_updateworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_tasks = &sas_dsname_tasks";
         %end;
         %if (&sas_dsname_parameters NE _lsaf_updateworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_parameters = &sas_dsname_parameters";
         %end;
         %if (&sas_dsname_inputs NE _lsaf_updateworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_inputs = &sas_dsname_inputs";
         %end;
         %if (&sas_dsname_outputs NE _lsaf_updateworkspacejob_ds_notprovided_) %then
         %do;
            put logMsgPrefix " sas_dsname_outputs = &sas_dsname_outputs";
         %end;
      %end;

      %***********************************;
      %* Set the macro variables         *;
      %***********************************;
      call symputx("_lsafRC_", returnCode);
      call symputx("_lsafMsg_", message);

   run;

   options &notesOption;

   %return;

   %PRINT_DSERROR_LOG_MSGS:
      %put &logMsgPrefix  &macroName ;
      %put &logMsgPrefix _lsafRC_= &_lsafRC_ ;
      %put &logMsgPrefix _lsafMsg_= &_lsafMsg_ ;
      %put;

   options &notesOption;

%mend lsaf_updateworkspacejob;
