%* lsaf_validate_sasdataset.sas                                                        *;
%*                                                                                     *;
%* Validates the data set sent in by the user to the SAS macros.  Tests that           *;
%* - data set has a validate SAS name                                                  *;
%* - exists                                                                            *;
%* - is not empty (has observations)                                                   *;
%* - includes the required variables                                                   *;
%* - required variables are of  the correct variable type (character or numeric)       *;
%*                                                                                     *;
%* Syntax:                                                                             *;
%* %LSAF_VALIDATE_SASDATASET(SAS_DSN=SAS-data-set                                      *;
%*            <, CALLINGMACRO=calling-macro, REQUIRED_VARIABLES=required-var-list,     *;
%*            REQUIRED_NUMERIC_VARIABLES=required-numeric-list>,                       *;
%*            ALLOW_EMPTY_DATASET=allow-empty-dataset);                                *;
%*                                                                                     *;
%* @macvar _lsafRC_  the return code.                                                  *;
%* @macvar _lsafMsg_ the return message.                                               *;
%*                                                                                     *;
%* @param sas_dsn - required - the name of the data set that is being validated.       *;
%* @param callingMacro - optional - the macro calling this one.                        *;
%* @param required_variables - optional - a space-delimited list of data set variables *;
%*             that are required to be present in the data set.                        *;
%* @param required_numeric_variables - optional - a space-delimited list of the        *;
%*             required variables that should be of type numeric. If a required        *;
%*             variable is not included in this list, it will be validated as a        *;
%*             character.                                                              *;
%* @param all_empty_dataset - optional - determines whether an error should be         *;
%*             returned if the input data set has 0 obs. The default is 0 (do not      *;
%*             allow 0 obs).                                                           *;
%*                                                                                     *;
%* @macrotype LSAF                                                                     *;
%* @since 1.2                                                                          *;
%* @exposure internal                                                                  *;
%*                                                                                     *;
%* @history 2013-08-30  updates for standardization and coding conventions.            *;
%* @history 2014-03-28  decoupled from updateacls.                                     *;
%* @history 2016-08-26  rebrand as LSAF                                                *;
%* @history 2016-11-07  added parameter allow_empty_datasets                           *;
%* @history 2017-01-09  updated documentation for allow_empty_datasets                 *;
%*                                                                                     *;

%macro lsaf_validate_sasdataset(
   sas_dsn=,
   callingMacro=,
   required_variables=,
   required_numeric_variables=%str(),
   allow_empty_datasets=0
   ) / des='Validates a SAS data set.';

   %put NOTE: &callingMacro.: Validating data set &sas_dsn;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_=%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %****************************************;
   %* VERIFY: data set name                *;
   %****************************************;
   %if (%qcmpres(&sas_dsn) eq %str()) %then
   %do;
      %let _lsafMsg_= %str(Input data set name was not provided.);
      %return;
   %end;

   %let validdsn=0;

   data _null_;
      datasetName="&sas_dsn";
      firstchar=ksubstr(kleft(datasetName), 1, 1);
      if ((length(datasetName) > 32) or (anydigit(firstchar))) then
      do;
         call symput("validdsn", put(1, best.));
      end;
   run;

   %if (&validdsn eq 1) %then
   %do;
      %let _lsafMsg_= %str(Input data set name is not valid.);
      %return;
   %end;

   %****************************************;
   %* VERIFY: data set exists              *;
   %****************************************;
   %if (%sysfunc(exist(&sas_dsn))=0) %then
   %do;
      %let _lsafMsg_= %str(Input data set &sas_dsn does not exist. );
      %return;
   %end;

   %local dsid;
   %let DSID=%sysfunc(OPEN(&sas_dsn., IN));

   %****************************************;
   %* VERIFY: data set has observations    *;
   %****************************************;
   %if (&allow_empty_datasets EQ 0) %then
   %do;
      %local nobs dsrc;
      %let RECORDS=%sysfunc(ATTRN(&DSID, NLOBS));
      %if (%eval(&RECORDS eq 0)) %then
      %do;
         %let _lsafMsg_= %str(Input data set &sas_dsn has no observations.);
         %let dsrc=%sysfunc(CLOSE(&DSID));
         %return;
      %end;
   %end;

   %********************************************************************;
   %* VERIFY: required parameters exist and are of the correct type.
   %********************************************************************;
   %if (%qcmpres(&required_variables) eq %str()) %then
      %GOTO END_VALIDATION;

   %local varCnt varName varPos missingVars invalidVars DSINVALID;
   %let varCnt=1;
   %let varName=_null_;
   %let missingVars=%str();
   %let invalidCharVars=%str();
   %let invalidNumVars=%str();
   %let DSINVALID=0;

   %do %until (&varName=%str());
      %let varName=%scan(&required_variables, &varCnt);
      %if (&varName eq %str()) %then
         %goto CONTINUE;

      %let varPos=%sysfunc(varnum(&DSID,&varName));

      %************************************;
      %* variable is missing              *;
      %************************************;
      %if (%eval(&varPos < 1)) %then
      %do;
         %let missingVars=%str(%trim(&missingVars) %trim(&varName));
         %let DSINVALID=1;
         %goto CONTINUE;
      %end;

      %let actualVarType = %sysfunc( vartype(&DSID, &varPos) );

      %if (%index(&required_numeric_variables, &varName) > 0) %then
      %do;
         %******************************************;
         %* variable that should be NUMERIC is not *;
         %******************************************;
         %if (&actualVarType ne N) %then
         %do;
            %let invalidNumVars=%str(%trim(&invalidNumVars) %trim(&varName));
            %let DSINVALID=1;
         %end;
      %end;
      %else %if (&actualVarType ne C) %then
      %do;
         %*********************************************;
         %*  variable that should be CHARACTER is not *;
         %*********************************************;
         %let invalidCharVars=%str(%trim(&invalidCharVars) %trim(&varName));
         %let DSINVALID=1;
      %end;

      %CONTINUE:
      %let varCnt=%eval(&varCnt+1);
   %end;

   %let rc=%sysfunc(CLOSE(&DSID));

   %if (&DSINVALID ne 0) %then
   %do;
      %**************************************************************************************;
      %* only setting one message so giving missing variables precedence over variable type *;
      %**************************************************************************************;
      %if (&missingVars ne %str()) %then
      %do;
         %let _lsafMsg_= %cmpres(Data set &sas_dsn is missing required variables: &missingVars);
         %put;
      %end;
      %else %if (&invalidNumVars ne %str()) %then
      %do;
         %let _lsafMsg_= %cmpres(Variables in data set &sas_dsn that must be numeric are not: &invalidNumVars);
      %end;
      %else %if (&invalidCharVars ne %str()) %then
      %do;
         %let _lsafMsg_= %cmpres(Variables in data set &sas_dsn that must be character are not: &invalidCharVars);
      %end;
      %return;
   %end;

   %END_VALIDATION:
   %let _lsafRC_=%str(0);
   %let _lsafMsg_=%str(Data set &sas_dsn is valid for &callingMacro..);

%mend lsaf_validate_sasdataset;
