%* sasdrugdev_deleterbitems                                                                        *;
%*                                                                                                 *;
%* Permanently deletes items from the SAS Drug Development recycle bin using a data set as         *;
%* input. Returns a SAS data set that contains the paths for the items that were permanently       *;
%* deleted from the SAS Drug Development recycle bin.                                              *;
%*                                                                                                 *;
%* The items listed in the input data set will be permanently removed from the recycle bin         *;
%* provided that the user has the Permanent Delete Items role privilege at the organizational      *;
%* context.  No error message will be provided for objects that could not be permanently deleted.  *;
%*                                                                                                 *;
%* A path created and deleted multiple times from the SAS Drug Development repository will appear  *;
%* multiple times in the recycle bin. Therefore, the items must be referenced by their unique      *;
%* identifier. The identifier can be retrieved using the sasdrugdev_getrbitems macro.              *;
%*                                                                                                 *;
%* The output data set will contain one row for each path that was permanently deleted from the    *;
%* recycle bin.  If multiple items with the same path were deleted, the path will only be listed   *;
%* once. The paths for each object deleted from a container (e.g. project, folder) will be listed  *;
%* separately in the output data set.                                                              *;
%*                                                                                                 *;
%* Syntax:                                                                                         *;
%* %SASDRUGDEV_deleterbitems(SAS_DSNAME=SAS-data-set, SAS_RESULT_DSNAME=SAS-data-set);             *;
%*                                                                                                 *;
%* Example Code:                                                                                   *;
%* Delete everything a specific user deleted.                                                      *;
%*                                                                                                 *;
%* * start code example;                                                                           *;
%* %sasdrugdev_getrbitems(sas_dsname=sddGetRBItems);                                               *;
%*                                                                                                 *;
%* data deleterbitems;                                                                             *;
%* set sddGetRBItems;                                                                              *;
%* if (deletedBy="myuserid");                                                                      *;
%* run;                                                                                            *;
%*                                                                                                 *;
%* %sasdrugdev_deleterbitems(sas_dsname=deleterbitems, sas_result_dsname=deletedpaths);            *;
%* * end code example;                                                                             *;
%*                                                                                                 *;
%* @macvar _sddRC_  the return code.                                                               *;
%* @macvar _sddMsg_ the return message.                                                            *;
%*                                                                                                 *;
%* @param sas_dsname - required - the name of a SAS data set containing the identifiers for the    *;
%*            items to be deleted from the recycle bin.  The data set must contain, at minimum,    *;
%*            the variables listed below.  Any additional variables will be ignored.               *;
%*            - identifier: The unique SAS Drug Development object identifier for the item to be   *;
%*              permanently deleted.                                                               *;
%* @param sas_result_dsname - optional - the name of the output SAS data set that will contain the *;
%*            deleted paths as libref.dataset. The default value is WORK.SDDDELETEDRBITEMS. The    *;
%*            data set contains a row for each deleted path and columns with the following names.  *;
%*            The data set will be sorted by path.                                                 *;
%*            - name: The name of the item.                                                        *;
%*            - path: The path of the item.                                                        *;
%*                                                                                                 *;
%*                                                                                                 *;
%* @macrotype SDD                                                                                  *;
%* @since 1.4                                                                                      *;
%* @exposure external                                                                              *;
%* @deprecated Replaced by lsaf_deleterbitems, effective with SAS LSAF Macro 1.5                   *;
%*                                                                                                 *;
%* @history 2013-03-27  initial code                                                               *;
%* @history 2016-08-30  rebrand for LSAF                                                           *;
%*                                                                                                 *;

%macro sasdrugdev_deleterbitems(
  sas_dsname=,
  sas_result_dsname=work.sddDeleteRBItems
  ) /des='Deletes the specified items from the SAS Drug Development recycle bin.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %global _sddMsg_
          _sddRC_
  ;

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_deleterbitems(
    sas_dsname=&sas_dsname,
    sas_result_dsname=&sas_result_dsname
  );

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=,
    lsaf_answer_val=
  );

%mend sasdrugdev_deleterbitems;
