%* sasdrugdev_getacls                                                                            *;
%*                                                                                               *;
%* Returns a SAS data set that contains the access control list for objects in the SAS Drug      *;
%* Development repository.                                                                       *;
%*                                                                                               *;
%* For container objects, both the DEFAULT and CURRENT access control lists are returned with    *;
%* the exception of analyses. For non-container objects and Analyses, only the CURRENT access    *;
%* control lists is returned.                                                                    *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %SASDRUGDEV_GETACLS(SDD_PATH=sdd-path, <SAS_DSNAME=SAS-data-set,                              *;
%* SDD_RECURSIVE=recursion-level>);                                                              *;
%*                                                                                               *;
%* @macvar _sddRC_  the return code.                                                             *;
%* @macvar _sddMsg_ the return message.                                                          *;
%*                                                                                               *;
%* @param sdd_path - required - the path of the content object in the SAS Drug Development       *;
%*            repository.  This can be either a file or a container.                             *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the access control lists specified as libref.dataset. The default     *;
%*            value is WORK.SDDGETACL. The data set contains a row for each access control       *;
%*            entry, sorted by path and aclType, and columns with the following names:           *;
%*            - name: The name of the object.                                                    *;
%*            - path: The path to the object.                                                    *;
%*            - objectType: Describes the type of the object (e.g. folder, data set, etc.)       *;
%*            - isContainer: A numeric flag that indicates whether the object is a container     *;
%*                  object (0=no, 1=yes).                                                        *;
%*            - owner: The user ID of the user who owns the object.                              *;
%*            - aclType: Will be either DEFAULT or CURRENT.  Analyses and files only have        *;
%*                  CURRENT acls.  All other containers have both DEFAULT and CURRENT acls.      *;
%*            - aclPrincipalName: If the aclPrincipalType is ACLOWNER this will be Owner.  If    *;
%*                  the aclPrincipalType is ACLMEMBERS this will be Members.  If the             *;
%*                  aclPrincipalType is USER this will be the user ID of the user.  If the       *;
%*                  aclPrincipalType is GROUP this will be the name of the group.                *;
%*            - aclPrincipalType: Specifies if the principal is a ACLOWNER, ACLMEMBERS, USER or  *;
%*                  GROUP.                                                                       *;
%*            - grpSrcCtxt: For group principals, specifies the context path where the group     *;
%*                  was defined.                                                                 *;
%*            - adminPermission: Permission to administer the object.                            *;
%*            - readPermission: Permission to read the object.                                   *;
%*            - writePropPerm: Permission to update the object properties.                       *;
%*            - writeContentPerm: Permission to update the object content.                       *;
%*            - deletePermission: Permission to delete the object.                               *;
%*                                                                                               *;
%*            Permission values are specified as follows.                                        *;
%*             1 = Allowed                                                                       *;
%*            -1 = Denied                                                                        *;
%*             0 = Not specified (Inherited)                                                     *;
%*                                                                                               *;
%* @param sdd_recursive - conditional - the level of recursion when the path is a container.     *;
%*            Valid values are 0, 1, 2, or 99 as described below. The default value is 0.  For   *;
%*            files, the value will be ignored.                                                  *;
%*                                                                                               *;
%*             0 - Returns acls for the input container only.                                    *;
%*             1 - Returns acls for the container and all objects within the container, but not  *;
%*                 sub containers.                                                               *;
%*             2 - Returns acls for the container, all objects within the container, and one     *;
%*                 level of sub containers.                                                      *;
%*            99 - Returns acls for the container and all of the sub containers and objects      *;
%*                 (traverses the entire tree).                                                  *;
%*                                                                                               *;
%* @macrotype SDD                                                                                *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%* @deprecated Replaced by lsaf_getacls, effective with SAS LSAF Macro 1.5                       *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-30  rebrand for LSAF                                                         *;
%*                                                                                               *;

%macro sasdrugdev_getacls(
  sdd_path=,
  sas_dsname=work.sddGetACLs,
  sdd_recursive=0
  ) /des='Get a list of the permissions for objects in the SAS Drug Development repository.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %global _sddMsg_
          _sddRC_
  ;

  %let _sddRC_ =%str(-999);
  %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_getacls(
    lsaf_path=&sdd_path,
    sas_dsname=&sas_dsname,
    lsaf_recursive=&sdd_recursive
  );

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=,
    lsaf_answer_val=
  );

%mend sasdrugdev_getacls ;
