%* sasdrugdev_getchildren                                                                        *;
%*                                                                                               *;
%* Returns a SAS data set that contains the metadata for objects within a container in the SAS   *;
%* Drug Development repository.                                                                  *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %SASDRUGDEV_GETCHILDREN(SDD_PATH=sdd-path <, SDD_RECURSIVE=recursion-level,                   *;
%* SAS_DSNAME=SAS-data-set>);                                                                    *;
%*                                                                                               *;
%* @macvar _sddRC_  the return code.                                                             *;
%* @macvar _sddMsg_ the return message.                                                          *;
%*                                                                                               *;
%* @param sdd_path - required - the path of the container object in the SAS Drug Development     *;
%*            repository.                                                                        *;
%*                                                                                               *;
%* @param sas_dsname - optional - the name of the output SAS data set that will contain the      *;
%*            metadata for the objects specified as libref.dataset. The default value is         *;
%*            WORK.SDDGETCHILDREN. The data set contains a row for each object in the list and   *;
%*            columns with the following names.  The data set will be sorted by path.            *;
%*            - name: The name of the object.                                                    *;
%*            - path: The path of the object.                                                    *;
%*            - objectType: Describes the type of the object (e.g. folder, data set, etc.)       *;
%*            - description: The description of the object.                                      *;
%*            - isContainer: A numeric flag that indicates whether the object is a container     *;
%*                  object (0=no, 1=yes).                                                        *;
%*            - isVersioned: A numeric flag that indicates whether the object is currently       *;
%*                  versioned (0=no, 1=yes).                                                     *;
%*            - versionLabel: The label associated with this specific version.                   *;
%*            - isCheckedOut: A  numeric flag that indicates whether the object is currently     *;
%*                  checked out (0=no, 1=yes).                                                   *;
%*            - isLocked: A numeric flag that indicates whether the object is currently locked   *;
%*                  (0=no, 1=yes).                                                               *;
%*            - size: The size of the object in bytes (will be 0 for containers).                *;
%*            - contentType: The content or MIME type of object.                                 *;
%*            - createdBy: The user ID that created the object.                                  *;
%*            - created: The full date and time when the object was created. This value is       *;
%*                  represented as a String.                                                     *;
%*            - dateCreated: The date when the object was created as a SAS Datetime format.      *;
%*            - lastModifiedBy: The user ID that last modified the object.                       *;
%*            - lastModified: The date when the object was last modified.  This value is         *;
%*                  represented as a String.                                                     *;
%*            - dateLastModified: The date when the object was last modified as a SAS Datetime   *;
%*                  format.                                                                      *;
%*            - propertiesLastModifiedBy: The user ID that last modified the properties for the  *;
%*                  object.                                                                      *;
%*            - propertiesLastModified: The date when the object properties were last modified.  *;
%*                  This value is represented as a String.                                       *;
%*            - datePropertiesLastModified: The date when the object properties were last        *;
%*                  modified as a SAS Datetime format.                                           *;
%*            - state: This variable is for an Analysis or Project object only.  It indicates    *;
%*                  if the state is DEV, PROD, or CLOSED.                                        *;
%*            - stateComment: This variable is for Analysis or Project only and provides any     *;
%*                  comments attached by a user when changing the state of the project or        *;
%*                  analysis.                                                                    *;
%*            - isSigned: Indicates if the object has electronic signatures attached to it.      *;
%*                                                                                               *;
%* @param sdd_recursive - optional - the level of recursion when the path is a container.        *;
%*            Valid values are 0, 1, 2, or 99 as described below. The default value is 0.  For   *;
%*            files, the value will be ignored.                                                  *;
%*                                                                                               *;
%*             0 - Returns the container only.                                                   *;
%*             1 - Returns the container and its immediate children.                             *;
%*             2 - Returns the container, its immediate children, and all objects within one     *;
%*                 level of the containers subfolders.                                           *;
%*            99 - Returns the container and all of the sub containers and objects               *;
%*                 (traverses the entire tree).                                                  *;
%* @macrotype SDD                                                                                *;
%* @since 1.1                                                                                    *;
%* @exposure external                                                                            *;
%* @deprecated Replaced by lsaf_getchildren, effective with SAS LSAF Macro 1.5                   *;
%*                                                                                               *;
%* @history 2013-06-28  version 1.2 update - path length                                         *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2014-05-07  updated documentation and fixed data set processing.                     *;
%* @history 2016-08-30  rebrand for LSAF                                                         *;
%*                                                                                               *;

%macro sasdrugdev_getchildren(
  sdd_path=,
  sas_dsname=work.sddGetChildren,
  sdd_recursive=0
  ) /des='Get a list of the objects in a folder in SAS Drug Development.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %global _sddMsg_
          _sddRC_
  ;

  %let _sddRC_ =%str(-999);
  %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_getchildren(
    lsaf_path=&sdd_path,
    sas_dsname=&sas_dsname,
    lsaf_recursive=&sdd_recursive
  );

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=,
    lsaf_answer_val=
  );

%mend sasdrugdev_getchildren ;
