%* sasdrugdev_isgroupmember                                                                      *;
%*                                                                                               *;
%* Determines whether a user or group is a member of the group in the specified organization,    *;
%* project, or analysis.                                                                         *;
%*                                                                                               *;
%* The SAS macro variable _sddIsGroupMember_ is set to a value of 1 when the specified user or   *;
%* group is a member of the group, 0 if it is not. In the event of an incorrect call to the      *;
%* macro which results in an error, the return value of _sddIsGroupMember_ will be set to -1.    *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %SASDRUGDEV_ISGROUPMEMBER(SDD_PATH=sdd-path, SDD_GROUP=sdd-group-name,                        *;
%* SDD_MEMBER=sdd-member-identifier, <, SDD_TYPE=sdd-type, SDD_GROUP_CONTEXT=sdd-group-context>);*;
%*                                                                                               *;
%* @macvar _sddRC_  the return code.                                                             *;
%* @macvar _sddMsg_ the return message.                                                          *;
%* @macvar _sddIsGroupMember_ indicates if the user or group is a member of the specified group. *;
%*                                                                                               *;
%* @param sdd_path - required -  the path of the context object (organization, project, or       *;
%*            analysis) where the group is assigned.                                             *;
%* @param sdd_group - required - the name of the group.                                          *;
%* @param sdd_member - required - the user ID or group name of the member.                       *;
%* @param sdd_type - conditional - indicates the type of member. Valid values are USER and GROUP *;
%*            (case-insensitive).  The default value is USER.                                    *;
%* @param sdd_group_context - conditional - specifies the context in which the member group is   *;
%*            defined as a path (e.g., /SAS). The parameter is required if SDD_TYPE=GROUP.       *;
%*            Otherwise, it will be ignored.                                                     *;
%* @param sdd_includeImplicit - optional -  indicates whether or not to include implied members, *;
%*            i.e., members of groups within groups. The valid values are 0 and 1 to exclude or  *;
%*            includeimplied memberships respectively. The default is 0.                         *;
%*                                                                                               *;
%* @macrotype SDD                                                                                *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%* @deprecated Replaced by lsaf_isgroupmember, effective with SAS LSAF Macro 1.5                 *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-31  rebrand for LSAF                                                         *;
%* @history 2016-01-18  added includeImplicit flag.                                              *;
%*                                                                                               *;

%macro sasdrugdev_isgroupmember(
  sdd_path=,
  sdd_group=,
  sdd_member=,
  sdd_type=USER,
  sdd_group_context=,
  sdd_includeImplicit=
  ) /des='Is the user or group a member of the group in the specified context?';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %global _sddIsGroupMember_
          _sddMsg_
          _sddRC_
  ;

  %let _sddRC_ =%str(-999);
  %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_isgroupmember(
    lsaf_path=&sdd_path,
    lsaf_group=&sdd_group,
    lsaf_member=&sdd_member,
    lsaf_type=&sdd_type,
    lsaf_group_context=&sdd_group_context,
    lsaf_includeImplicit=&sdd_includeImplicit
  );

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=_sddIsGroupMember_,
    lsaf_answer_val=&_lsafIsGroupMember_
  );

%mend sasdrugdev_isgroupmember ;
