%* sasdrugdev_submitjob                                                                          *;
%*                                                                                               *;
%* Submits a SAS Drug Development repository job for immediate execution and sets the SAS macro  *;
%* variable _sddJobSubmissionId_ to the identifier for the job submission. If the job cannot     *;
%* be submitted, _sddJobSubmissionId_ will be set to blank.                                      *;
%*                                                                                               *;
%* A data set representing the job parameters whose values are to be overridden for the current  *;
%* job submission can be sent into the call of sasdrugdev_submitjob. If no job parameters data   *;
%* set is sent in, the default values for all job parameters will be used.                       *;
%* The output data set from the macro sasdrugdev_getjobparameters contains all the data relevant *;
%* to the parameters for a specific job and can be used as a source for the input parameters data*;
%* set for the sasdrugdev_submitjob macro. Modify the data set with any desired changes prior to *;
%* calling sasdrugdev_submitjob.                                                                 *;
%* - Rename the defaultValue variable to "value" or add a variable with the name "value" to      *;
%*   indicate the values to use for the current submission.                                      *;
%* - Default job parameter values will be used for missing parameters (deleted rows)             *;
%* - Indicating an empty string for a parameter value will result in using a blank as the        *;
%*   parameter value for the current submission.                                                 *;
%* - Added parameters will be ignored.                                                           *;
%* - The parameters in the input data set are matched with the job parameters by name and type.  *;
%*   If no match is found for a parameter, its default value will be used.                       *;
%* - Data set rows with duplicate values for parameter name will cause an execution error.       *;
%* - DATE job parameter values must be in SAS datetime18. format (e.g., 02NOV2013:09:33:22), or  *;
%*   in SAS E8601DZ25. format (e.g., 2013-11-02T09:33:22-04:00).  For SAS datetime18. format, an *;
%*   offset of zero hours from GMT will be used.  To specify an offset from GMT, use the         *;
%*   E8601DZ25. format.  Date values in formats other than the two described above will result   *;
%*   in an execution error.                                                                      *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %SASDRUGDEV_SUBMITJOB(SDD_PATH=sdd-path <, SDD_VERSION=sdd-version,                           *;
%*                       SAS_DSNAME=SAS-data-set>);                                              *;
%*                                                                                               *;
%* @macvar _sddRC_  the return code.                                                             *;
%* @macvar _sddMsg_ the return message.                                                          *;
%*                                                                                               *;
%* @param sdd_path - required -the path of the job in the SAS Drug Development repository.       *;
%* @param sdd_version - optional - a specific version number of the job. If not specified,       *;
%*            the latest version of the job will be submitted. Specifying a version for a        *;
%*            non-versioned file will cause an error.                                            *;
%* @param sas_dsname - optional - the name of the SAS data set containing the job parameter      *;
%*            information specified as libref.dataset. If not specified, the default job         *;
%*            parameters values will be used.  The data set must contain, at minimum, the        *;
%*            character variables listed below.  Any additional variables will be ignored.       *;
%*            - name: The name of the parameter.                                                 *;
%*            - type: The parameter type.  Valid values are CHARACTER, CHARACTER_MASKED,         *;
%*                  NUMERIC, DATE, FILE, and FOLDER.                                             *;
%*            - value: The value of the parameter to be used in the current job submission.      *;
%*            - fileVersion: The file version to be used if the parameter is of type FILE.       *;
%*                  For versioned files, a blank value will result in the latest file version    *;
%*                  used.  For nonversioned files, the value is ignored.                         *;
%*                                                                                               *;
%* @macrotype SDD                                                                                *;
%* @since 1.4                                                                                    *;
%* @exposure external                                                                            *;
%* @deprecated Replaced by lsaf_submitjob, effective with SAS LSAF Macro 1.5                     *;
%*                                                                                               *;
%* @history 2014-04-09  initial coding                                                           *;
%* @history 2016-08-31  rebrand for LSAF                                                         *;
%*                                                                                               *;

%macro sasdrugdev_submitjob(
   sdd_path=,
   sdd_version=,
   sas_dsname=
   ) /des='Submits a repository job for execution.';

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %global _sddJobSubmissionId_
           _sddMsg_
           _sddRC_
   ;

   %let _sddJobSubmissionId_=;
   %let _sddRC_ =%str(-999);
   %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_submitjob(
    lsaf_path=&sdd_path,
    lsaf_version=&sdd_version,
    sas_dsname=&sas_dsname
  );

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=_sddJobSubmissionId_,
    lsaf_answer_val=&_lsafJobSubmissionId_
  );

%mend sasdrugdev_submitjob;
