%* sasdrugdev_updateacls                                                                         *;
%*                                                                                               *;
%* Sets permissions on SAS Drug Development objects using a data set as input.                   *;
%*                                                                                               *;
%* This macro sets access permissions on repository objects using a SAS data set as input. The   *;
%* permission lists (known as Access Control Lists or ACLs) for multiple objects can be modified *;
%* using a single data set.  Access permissions are applied only to objects in the repository    *;
%* within the organization. There are two types of access permission lists: Current and Default. *;
%* The Current access permissions are applied to an individual object.  The Default access       *;
%* permissions apply only to container objects (e.g. folders, projects) and define the           *;
%* permissions that will be inherited by any new objects added to the container.  In the context *;
%* of this macro the access permission type is referred to as aclType.  All records in the input *;
%* data set that pertain to the same repository object and aclType will be processed together as *;
%* a single ACL.                                                                                 *;
%*                                                                                               *;
%* A set of permissions for a specific repository object for an individual principal within an   *;
%* access permission type are referred to as an Access Control Entry (ACE).  An ACE is           *;
%* represented as a single row in the input data set. An ACE is defined by the following         *;
%* characteristics:                                                                              *;
%* - path: Full path of the repository object (file or container) to which the permissions are   *;
%*   being assigned.                                                                             *;
%* - acl type: The access permission type is being assigned (DEFAULT or CURRENT)                 *;
%* - acl principal type: Identifies the type of the principal. The principal type can be Owner,  *;
%*   Members, User, or Group.                                                                    *;
%* - acl principal name: The name of the user or group to assign the permission to.              *;
%* - set of permissions: The specific permissions being assigned (e.g. administration, read,     *;
%*   properties write, content write, delete).                                                   *;
%*                                                                                               *;
%* The output data set from the sasdrugdev_getacls macro contains all the necessary variables    *;
%* and data needed as the input data set for the sasdrugdev_updateACLs macro. Modify the data    *;
%* set with any changes prior to calling sasdrugdev_updateacls.                                  *;
%* - To add a principal with specific access permissions to a repository object, add a new row   *;
%*   to the data set for that principal.                                                         *;
%* - To modify the access permissions for a principal already associated with a repository       *;
%*   object, edit the appropriate row in the data set.                                           *;
%* - To remove a principal with specific access permissions to a repository object, delete the   *;
%*   appropriate row from the data set. Note that the Owner and Members access control entries   *;
%*   cannot be removed from the object. Removing them from the data set will cause a failure.    *;
%* - To maintain the existing permissions for a principal already associated with a repository   *;
%*   object, leave the record as it is in the data set. The record must remain otherwise the     *;
%*   access permissions for this principal will be removed.                                      *;
%*                                                                                               *;
%* Failure and Message Handling:                                                                 *;
%* The macro performs a series of validation procedures to determine that the input data set has *;
%* the necessary structure for processing. If any of these validation procedures fail, an        *;
%* appropriate failure message is printed to the log and the macro stops additional processing.  *;
%*                                                                                               *;
%* In addition to other modifications, the macro sorts the input data set and stores the modified*;
%* data in a temporary data set called WORK.__SASMACRO_ACLTEMP__. In the event that a failure    *;
%* occurs during processing this temporary data set is retained as a reference. If all updates   *;
%* are performed successfully, this data set will be deleted from the work library.  A message   *;
%* will be printed to the log reporting whether the data set was retained or not. Regardless of  *;
%* success or failure of previous runs, this data set will be deleted at the beginning of each   *;
%* new run of the macro.  The data values and sort order of the original input data set are      *;
%* maintained.                                                                                   *;
%*                                                                                               *;
%* All records pertaining to the same repository object and aclType are processed as a single    *;
%* access control list by the macro. Messages will be printed to the log as each ACL is          *;
%* processed. If the update of an ACL fails, the macro will stop processing.                     *;
%*                                                                                               *;
%* If a failure occurs while processing a specific access control entry (i.e. a principal), the  *;
%* macro will stop processing and a failure message will be written to the log including the     *;
%* observation number for the entry that failed.  The observation number refers to the           *;
%* observation in the interim data set (WORK.__SASMACRO_ACLTEMP__) created by the macro.         *;
%* Successful updates to ACLs made prior to the observation containing the failure will be       *;
%* maintained in SAS Drug Development. Changes made to access permissions for entries that are   *;
%* part of the same ACL that contained the observation with the failure will be lost.            *;
%*                                                                                               *;
%* Valid values for permissions are as follows.  All values must be of type numeric.             *;
%*  1 for GRANT                                                                                  *;
%* -1 for DENY                                                                                   *;
%*  0 for inherited.                                                                             *;
%*                                                                                               *;
%* Syntax:                                                                                       *;
%* %SASDRUGDEV_UPDATEACLS(SAS_DSNAME=SAS-data-set);                                              *;
%*                                                                                               *;
%* Example Code:                                                                                 *;
%* Get permissions for a folder:                                                                 *;
%*                                                                                               *;
%* * start code example;                                                                         *;
%* %sasdrugdev_getacls(sdd_path=%str(/SAS/Study1/Files/folder1), sdd_recursive=0);               *;
%*                                                                                               *;
%* data UpdateACLS;                                                                              *;
%* set sddGetACLS end=eof;                                                                       *;
%*                                                                                               *;
%* output;                                                                                       *;
%*                                                                                               *;
%* if eof then do;                                                                               *;
%*   * Adding user permission under DEFAULT;                                                     *;
%*   aclType = "DEFAULT";                                                                        *;
%*   aclPrincipalType = "USER";                                                                  *;
%*   aclPrincipalName = "user1";                                                                 *;
%*   grpSrcCtxt = "";                                                                            *;
%*   adminPermission = 0;                                                                        *;
%*   readPermission = 1;                                                                         *;
%*   writePropPerm = 1;                                                                          *;
%*   writeContentPerm = 1;                                                                       *;
%*   deletePermission = 1;                                                                       *;
%*   output;                                                                                     *;
%*                                                                                               *;
%*   * Adding group permission under CURRENT;                                                    *;
%*   aclType = "CURRENT";                                                                        *;
%*   aclPrincipalType = "GROUP";                                                                 *;
%*   aclPrincipalName = "group1";                                                                *;
%*   grpSrcCtxt = "/SAS";                                                                        *;
%*   adminPermission = 0;                                                                        *;
%*   readPermission = 1;                                                                         *;
%*   writePropPerm = 1;                                                                          *;
%*   writeContentPerm = 1;                                                                       *;
%*   deletePermission = 1;                                                                       *;
%*   output;                                                                                     *;
%* end;                                                                                          *;
%*                                                                                               *;
%* run;                                                                                          *;
%*                                                                                               *;
%* %sasdrugdev_updateacls(sas_dsname=UpdateACLS);                                                *;
%* * end code example;                                                                           *;
%*                                                                                               *;
%* Note: Access Control Entries (ACEs) with values of 0 for all permission variables will be     *;
%*       removed from or will not be added to the ACL.                                           *;
%* Note: Users and groups being added as access control entries must be members in the context   *;
%*       (organization, project, analysis) containing the object being updated.                  *;
%*                                                                                               *;
%* @macvar _sddRC_  the return code.                                                             *;
%* @macvar _sddMsg_ the return message.                                                          *;
%*                                                                                               *;
%* @param sas_dsname - required - the name of a SAS data set containing the permission           *;
%*            information for all the objects to be modified specified as libref.dataset. The    *;
%*            data set must contain, at minimum, the variables listed below.  Any additional     *;
%*            variables will be ignored.                                                         *;
%*            - path: The case-sensitive path of the object including object name.               *;
%*            - aclType: Indicates which ACL to modify for the object. For Analysis and file     *;
%*                  objects, the variable must be present but the value will be ignored. For     *;
%*                  container objects, the value must be either DEFAULT or CURRENT               *;
%*                  (case-insensitive).                                                          *;
%*            - aclPrincipalType: Indicates the type of ACE being updated.                       *;
%*                  Values:  ACLOWNER | ACLMEMBERS | USER | GROUP (case-insensitive)             *;
%*                  Default: None                                                                *;
%*            - aclPrincipalName: The name of the principal ACE being updated or added.          *;
%*                  For aclPrincipalType ACLOWNER, this must have the value OWNER.               *;
%*                  For aclPrincipalType ACLMEMBERS, this must have the value MEMBERS.           *;
%*                  For aclPrincipalType USER, this must be a valid userid (case-sensitive).     *;
%*                  For aclPrincipalType GROUP, this must be a valid group name (case-sensitive).*;
%*            - grpSrcCtxt: The case-sensitive path of the context (organization, project, or    *;
%*                  analysis) where the group is defined if aclPrincipalType is GROUP. For       *;
%*                  aclPrincipalType GROUP, this value is required, otherwise the variable must  *;
%*                  be present but the value will be ignored.                                    *;
%*            - adminPermission: The permission to administer the object.                        *;
%*            - readPermission: The permission to view the object.                               *;
%*            - writePropPerm: The permission to update the properties of the object.            *;
%*            - writeContentPerm: The permission to write to the content of the object.          *;
%*            - deletePermission: The permission to the delete the object.                       *;
%*                                                                                               *;
%* @macrotype SDD                                                                                *;
%* @since 1.2                                                                                    *;
%* @exposure external                                                                            *;
%* @deprecated Replaced by lsaf_updateacls, effective with SAS LSAF Macro 1.5                    *;
%*                                                                                               *;
%* @history 2013-08-30  updates for standardization and coding conventions                       *;
%* @history 2016-08-31  rebrand for LSAF                                                         *;
%*                                                                                               *;

%macro sasdrugdev_updateacls(
  sas_dsname=
  ) /des='Set permissions on SAS Drug Development objects.';

  %****************************************;
  %* Initialize macro variables           *;
  %****************************************;
  %global _sddMsg_
          _sddRC_
  ;

  %let _sddRC_ =%str(-999);
  %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

  %****************************************;
  %* Call LSAF macro.                     *;
  %****************************************;
  %lsaf_updateacls(sas_dsname=&sas_dsname);

  %***************************************;
  %* Call LSAF transition code           *;
  %***************************************;
  %let _sddMacro_=&SYSMACRONAME;
  %lsaf_transition(
    sdd_macro_name=%lowcase(&_sddMacro_),
    sdd_answer_var=,
    lsaf_answer_val=
  );

%mend sasdrugdev_updateacls;
